@@ -1,29 +1,39 @@
 /*
- * Copyright (C), 2000-2007 by the monit project group.
- * All Rights Reserved.
+ * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 3 of the
- * License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 3.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
  */
 
 /**
- *  System independent device methods.
+ *  System independent filesystem methods.
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: device_common.c,v 1.9 2007/07/25 12:54:30 hauk Exp $
+ *  @version \$Id: device_common.c,v 1.13 2009/02/13 09:18:12 hauk Exp $
  *
  *  @file
  */
@@ -67,7 +77,7 @@
 /**
  * This function validates whether given object is valid for filesystem
  * informations statistics and stores path suitable for it in given
- * device information structure for later use. Filesystem must be mounted.
+ * filesystem information structure for later use. Filesystem must be mounted.
  *
  * Valid objects are file or directory that are part of requested
  * filesystem, block special device or mountpoint.
@@ -114,11 +124,11 @@ char *device_path(Info_T inf, char *object) {
  * given information structure.
  *
  * @param inf Information structure where resulting data will be stored
- * @param object  Identifies requested device - either file, directory,
+ * @param object  Identifies requested filesystem - either file, directory,
  *                block special device or mountpoint
  * @return        TRUE if informations were succesfully read otherwise FALSE
  */
-int device_usage(Info_T inf, char *object) {
+int filesystem_usage(Info_T inf, char *object) {
 
   ASSERT(inf);
   ASSERT(object);
@@ -130,7 +140,7 @@ int device_usage(Info_T inf, char *object) {
   /* save the previous filesystem flags */
   inf->_flags= inf->flags;
 
-  return device_usage_sysdep(inf);
+  return filesystem_usage_sysdep(inf);
 
 }
 
