@@ -1,29 +1,39 @@
 /*
- * Copyright (C), 2000-2007 by the monit project group.
- * All Rights Reserved.
+ * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 3 of the
- * License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 3.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
  */
 
 /**
- *  System dependent device methods.
+ *  System dependent filesystem methods.
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: sysdep_FREEBSD.c,v 1.18 2007/07/25 12:54:31 hauk Exp $
+ *  @version \$Id: sysdep_FREEBSD.c,v 1.22 2009/02/13 09:18:13 hauk Exp $
  *
  *  @file
  */
@@ -55,7 +65,7 @@
 
 /**
  * FreeBSD special block device mountpoint method. Filesystem must be mounted.
- * In the case of success, mountpoint is stored in device information
+ * In the case of success, mountpoint is stored in filesystem information
  * structure for later use.
  *
  * @param inf  Information structure where resulting data will be stored
@@ -70,7 +80,7 @@ char *device_mountpoint_sysdep(Info_T inf, char *blockdev) {
   ASSERT(blockdev);
 
   if(statfs(blockdev, &usage) != 0) {
-    LogError("%s: Error getting mountpoint for device '%s' -- %s\n",
+    LogError("%s: Error getting mountpoint for filesystem '%s' -- %s\n",
         prog, blockdev, STRERROR);
     return NULL;
   }
@@ -87,14 +97,14 @@ char *device_mountpoint_sysdep(Info_T inf, char *blockdev) {
  * @param inf Information structure where resulting data will be stored
  * @return        TRUE if informations were succesfully read otherwise FALSE
  */
-int device_usage_sysdep(Info_T inf) {
+int filesystem_usage_sysdep(Info_T inf) {
 
   struct statfs usage;
 
   ASSERT(inf);
 
   if(statfs(inf->mntpath, &usage) != 0) {
-    LogError("%s: Error getting usage statistics for device '%s' -- %s\n",
+    LogError("%s: Error getting usage statistics for filesystem '%s' -- %s\n",
         prog, inf->mntpath, STRERROR);
     return FALSE;
   }
