@@ -1,11 +1,8 @@
 /*
- * Copyright (C), 2000-2007 by the monit project group.
- * All Rights Reserved.
+ * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 3.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,6 +11,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
  */
 
 #include <config.h>
@@ -58,12 +69,20 @@
 #include <fcntl.h>
 #endif
 
+#ifdef HAVE_CRT_EXTERNS_H
+#include <crt_externs.h>
+#endif
+
 #include "monitor.h"
 
 #ifndef MAXPATHLEN
 #define MAXPATHLEN STRLEN
 #endif
 
+#ifdef DARWIN
+#define environ (*_NSGetEnviron())
+#endif
+
 /* Private prototypes */
 static void set_sandbox(void);
 static void set_environment(void);
@@ -74,7 +93,7 @@ static void set_environment(void);
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *
- *  @version \$Id: env.c,v 1.37 2007/07/25 12:54:28 hauk Exp $
+ *  @version \$Id: env.c,v 1.42 2009/02/13 09:18:06 hauk Exp $
  * 
  *  @file
  */
