@@ -1,11 +1,8 @@
 /*
- * Copyright (C), 2000-2007 by the monit project group.
- * All Rights Reserved.
+ * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 3.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,6 +11,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
  */
 
 #ifndef MONIT_EVENT_H
@@ -35,10 +46,13 @@
 #define EVENT_INVALID         0x400
 #define EVENT_DATA            0x800
 #define EVENT_EXEC            0x1000
-#define EVENT_CHANGED         0x2000
+#define EVENT_FSFLAG          0x2000
 #define EVENT_ICMP            0x4000
-#define EVENT_MATCH           0x8000
+#define EVENT_CONTENT         0x8000
 #define EVENT_INSTANCE        0x10000
+#define EVENT_ACTION          0x20000
+#define EVENT_PID             0x40000
+#define EVENT_PPID            0x80000
 #define EVENT_ALL             0xFFFFFFFF
 
 #define IS_EVENT_SET(value, mask) ((value & mask) != 0)
@@ -46,7 +60,9 @@
 typedef struct myeventtable {
   int id;
   char *description_failed;
-  char *description_passed;
+  char *description_succeeded;
+  char *description_changed;
+  char *description_changednot;
 } EventTable_T;
 
 extern EventTable_T Event_Table[];
@@ -64,7 +80,7 @@ extern EventTable_T Event_Table[];
  * 
  * @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  * @author Martin Pala, <martinp@tildeslash.com>
- * @version \$Id: event.h,v 1.28 2007/07/25 12:54:28 hauk Exp $
+ * @version \$Id: event.h,v 1.36 2009/03/24 19:49:05 martinp Exp $
  * @file
  */
 
@@ -76,8 +92,7 @@ extern EventTable_T Event_Table[];
  * @param action Description of the event action
  * @param s Optional message describing the event
  */
-void Event_post(Service_T service, long id, short state, EventAction_T action,
-  char *s, ...);
+void Event_post(Service_T service, long id, short state, EventAction_T action, char *s, ...);
 
 
 /**
@@ -117,7 +132,7 @@ int Event_get_source_type(Event_T E);
  * @param E An event object
  * @return The Event timestamp
  */
-time_t Event_get_collected(Event_T E);
+struct timeval *Event_get_collected(Event_T E);
 
 
 /**
