@@ -1,11 +1,8 @@
 /*
- * Copyright (C), 2000-2007 by the monit project group.
- * All Rights Reserved.
+ * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 3.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,6 +11,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
  */
 
 #include <config.h>
@@ -61,13 +72,13 @@
 #include "monitor.h"
 
 /**
- *  Utilities used for managing files used by monit.
+ *  Utilities for managing files used by monit.
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Christian Hopp, <chopp@iei.tu-clausthal.de>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: file.c,v 1.14 2007/08/10 21:34:33 martinp Exp $
+ *  @version \$Id: file.c,v 1.25 2009/02/13 09:18:06 hauk Exp $
  *
  *  @file
  */
@@ -82,7 +93,7 @@
 void File_init() {
   
   char pidfile[STRLEN];
-  char statefile[STRLEN];
+  char buf[STRLEN];
   
   /* Check if the pidfile was already set during configfile parsing */
   if(Run.pidfile == NULL) {
@@ -94,21 +105,38 @@ void File_init() {
     }
     Run.pidfile= xstrdup(pidfile);
   }
-  /* Set the location of the programs state file */
+
+  /* Set the location of monit's id file */
+  if(Run.idfile == NULL) {
+    snprintf(buf, STRLEN, "%s/.%s", Run.Env.home, MYIDFILE);
+    Run.idfile= xstrdup(buf);
+  }
+  Util_monitId(Run.idfile);
+
+  /* Set the location of monit's state file */
   if(Run.statefile == NULL) {
-    snprintf(statefile, STRLEN, "%s/.%s", Run.Env.home, MYSTATEFILE);
-    Run.statefile= xstrdup(statefile);
+    snprintf(buf, STRLEN, "%s/.%s", Run.Env.home, MYSTATEFILE);
+    Run.statefile= xstrdup(buf);
   }
   
 }
 
 
 /**
- * Finalize and remove temporary files
+ * Finalize and remove temporary files and make sure Monit id file exist
  */
 void File_finalize() {
   unlink(Run.pidfile);
-  unlink(Run.statefile);
+  // Make sure that Monit id file exist
+  if (! File_exist(Run.idfile)) {
+    FILE *f =  fopen(Run.idfile,"w");
+    if (! f) {
+      LogError("%s: Error opening Monit id file '%s' for writing -- %s\n", prog, Run.idfile, STRERROR);
+    } else {
+      fprintf(f, "%s\n", Run.id);
+      fclose(f);
+    }
+  }
 }
 
 
@@ -200,8 +228,7 @@ int File_createPidFile(char *pidfile) {
   umask(MYPIDMASK);
   unlink(pidfile);
   if ((F= fopen(pidfile,"w")) == (FILE *)NULL) {
-    LogError("%s: Error opening pidfile '%s' for writing -- %s\n",
-	prog, pidfile, STRERROR);
+    LogError("%s: Error opening pidfile '%s' for writing -- %s\n", prog, pidfile, STRERROR);
     return(FALSE);
   }
   fprintf(F, "%d\n", (int)getpid());
@@ -285,7 +312,7 @@ int File_exist(char *file) {
  * @param filename The filename of the checked file
  * @param description The description of the checked file
  * @param permmask The permission mask for the file
- * @return TRUE if the test passed otherwise FALSE
+ * @return TRUE if the test succeeded otherwise FALSE
  */
 int File_checkStat(char *filename, char *description, int permmask) {
 
@@ -362,7 +389,7 @@ int File_checkStat(char *filename, char *description, int permmask) {
  * specified mode.
  * @param path The fully qualified path to the directory
  * @param mode The permission for the directory
- * @return TRUE if the passed otherwise FALSE
+ * @return TRUE if the succeeded otherwise FALSE
  */
 int File_checkQueueDirectory(char *path, mode_t mode) {
 
@@ -396,7 +423,7 @@ int File_checkQueueDirectory(char *path, mode_t mode) {
  * Check the queue size limit.
  * @param path The fully qualified path to the directory
  * @param limit The queue limit
- * @return TRUE if the passed otherwise FALSE
+ * @return TRUE if the succeeded otherwise FALSE
  */
 int File_checkQueueLimit(char *path, int limit) {
 
@@ -404,10 +431,8 @@ int File_checkQueueLimit(char *path, int limit) {
   DIR           *dir = NULL;
   struct dirent *de = NULL;
 
-  if(limit <= 0) {
-    LogError("%s: event queue full\n", prog);
-    return FALSE;
-  }
+  if(limit < 0)
+    return TRUE;
 
   if(! (dir = opendir(path)) ) {
     LogError("%s: cannot open the directory %s -- %s\n", prog, path, STRERROR);
@@ -432,7 +457,7 @@ int File_checkQueueLimit(char *path, int limit) {
  * @param file Filedescriptor to write to
  * @param data Data to be written
  * @param size Size of the data to be written
- * @return TRUE if the passed otherwise FALSE
+ * @return TRUE if the succeeded otherwise FALSE
  */
 int File_writeQueue(FILE *file, void *data, int size) {
 
@@ -476,6 +501,7 @@ void *File_readQueue(FILE *file, int *size) {
   if(*size > 0) {
     data = xcalloc(1, *size);
     if(fread(data, 1, *size, file) != *size || ferror(file)) {
+      FREE(data);
       goto error;
     }
   }
