@@ -1,11 +1,8 @@
 /*
- * Copyright (C), 2000-2007 by the monit project group.
- * All Rights Reserved.
+ * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 3.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,6 +11,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
  */
 
 #include <config.h>
@@ -38,13 +49,12 @@ static void _gc_service_list(Service_T *);
 static void _gc_service(Service_T *);
 static void _gc_mail_server(MailServer_T *);
 static void _gcppl(Port_T *);
-static void _gcdevice(Device_T *);
+static void _gcfilesystem(Filesystem_T *);
 static void _gcicmp(Icmp_T *);
 static void _gcpql(Resource_T *);
 static void _gcptl(Timestamp_T *);
 static void _gc_action(Action_T *);
 static void _gc_eventaction(EventAction_T *);
-static void _gc_event(Event_T *);
 static void _gc_inf(Info_T *);
 static void _gcpdl(Dependant_T *);
 static void _gcso(Size_T *);
@@ -55,7 +65,7 @@ static void _gcuid(Uid_T *);
 static void _gcgid(Gid_T *);
 static void _gcgrc(Generic_T *);
 static void _gcath(Auth_T *);
-static void _gc_collector(Collector_T *);
+static void _gc_mmonit(Mmonit_T *);
 static void _gc_url(URL_T *);
 static void _gc_request(Request_T *);
 
@@ -66,7 +76,7 @@ static void _gc_request(Request_T *);
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: gc.c,v 1.73 2007/10/02 20:58:51 martinp Exp $
+ *  @version \$Id: gc.c,v 1.89 2009/02/13 09:18:07 hauk Exp $
  *
  *  @file
  */
@@ -98,11 +108,11 @@ void gc() {
   if(Run.mailservers)
     _gc_mail_server(&Run.mailservers);
 
-  if(Run.collectors)
-    _gc_collector(&Run.collectors);
+  if(Run.mmonits)
+    _gc_mmonit(&Run.mmonits);
 
   if(Run.eventlist)
-    _gc_event(&Run.eventlist);
+    gc_event(&Run.eventlist);
   
   FREE(Run.eventlist_dir);
   FREE(Run.mygroup);
@@ -113,6 +123,7 @@ void gc() {
   FREE(Run.MailFormat.from);
   FREE(Run.MailFormat.subject);
   FREE(Run.MailFormat.message);
+  FREE(Run.mail_hostname);
   
 }
 
@@ -133,7 +144,7 @@ void gc_mail_list(Mail_T *m) {
 }
 
 
-void _gccmd(Command_T *c) {
+void gccmd(Command_T *c) {
 
   int i;
   
@@ -146,6 +157,24 @@ void _gccmd(Command_T *c) {
 }
 
 
+void gc_event(Event_T *e) {
+
+  ASSERT(e&&*e);
+  
+  if((*e)->next)
+    gc_event(&(*e)->next);
+
+  pthread_mutex_destroy(&(*e)->mutex);
+
+  (*e)->action= NULL;
+  FREE((*e)->source);
+  FREE((*e)->group);
+  FREE((*e)->message);
+  FREE(*e);
+
+}
+
+
 /* ----------------------------------------------------------------- Private */
 
 
@@ -168,8 +197,8 @@ static void _gc_service(Service_T *s) {
   if((*s)->portlist)
     _gcppl(&(*s)->portlist);
 
-  if((*s)->devicelist)
-    _gcdevice(&(*s)->devicelist);
+  if((*s)->filesystemlist)
+    _gcfilesystem(&(*s)->filesystemlist);
 
   if((*s)->icmplist)
     _gcicmp(&(*s)->icmplist);
@@ -208,10 +237,10 @@ static void _gc_service(Service_T *s) {
     _gcpdl(&(*s)->dependantlist);
 
   if((*s)->start)
-    _gccmd(&(*s)->start);
+    gccmd(&(*s)->start);
 
   if((*s)->stop)
-    _gccmd(&(*s)->stop);
+    gccmd(&(*s)->stop);
 
   if((*s)->action_DATA)
     _gc_eventaction(&(*s)->action_DATA);
@@ -246,9 +275,13 @@ static void _gc_service(Service_T *s) {
   if((*s)->action_MONIT_RELOAD)
     _gc_eventaction(&(*s)->action_MONIT_RELOAD);
   
-  if((*s)->eventlist)
-    _gc_event(&(*s)->eventlist);
+  if((*s)->action_ACTION)
+    _gc_eventaction(&(*s)->action_ACTION);
   
+  if((*s)->eventlist)
+    gc_event(&(*s)->eventlist);
+  
+  FREE((*s)->token);
   FREE((*s)->name);
   FREE((*s)->group);
   FREE((*s)->path);
@@ -314,7 +347,7 @@ static void _gc_action(Action_T *a) {
   ASSERT(a&&*a);
   
   if((*a)->exec)
-    _gccmd(&(*a)->exec);
+    gccmd(&(*a)->exec);
   FREE(*a);
 
 }
@@ -325,25 +358,7 @@ static void _gc_eventaction(EventAction_T *e) {
   ASSERT(e&&*e);
   
   _gc_action(&(*e)->failed);
-  _gc_action(&(*e)->passed);
-  FREE(*e);
-
-}
-
-
-static void _gc_event(Event_T *e) {
-
-  ASSERT(e&&*e);
-  
-  if((*e)->next)
-    _gc_event(&(*e)->next);
-
-  pthread_mutex_destroy(&(*e)->mutex);
-
-  (*e)->action= NULL;
-  FREE((*e)->source);
-  FREE((*e)->group);
-  FREE((*e)->message);
+  _gc_action(&(*e)->succeeded);
   FREE(*e);
 
 }
@@ -373,12 +388,12 @@ static void _gcppl(Port_T *p) {
 }
 
 
-static void _gcdevice(Device_T *d) {
+static void _gcfilesystem(Filesystem_T *d) {
   
   ASSERT(d&&*d);
 
   if((*d)->next)
-    _gcdevice(&(*d)->next);
+    _gcfilesystem(&(*d)->next);
 
   if((*d)->action)
     _gc_eventaction(&(*d)->action);
@@ -422,9 +437,7 @@ static void _gc_inf(Info_T *i) {
 
   ASSERT(i);
 
-  if((*i)->cs_sum)
-    FREE((*i)->cs_sum);
-
+  FREE((*i)->cs_sum);
   FREE(*i);
 
 }
@@ -469,11 +482,8 @@ static void _gcmatch(Match_T *s) {
   if((*s)->action)
     _gc_eventaction(&(*s)->action);
 
-  if((*s)->match_path)
-      FREE((*s)->match_path);
-
-  if((*s)->match_string)
-      FREE((*s)->match_string);
+  FREE((*s)->match_path);
+  FREE((*s)->match_string);
 
 #ifdef HAVE_REGEX_H
   if((*s)->regex_comp) {
@@ -576,17 +586,18 @@ static void _gcath(Auth_T *c) {
 
   FREE((*c)->uname);
   FREE((*c)->passwd);
+  FREE((*c)->groupname);
   FREE(*c);
   
 }
 
 
-static void _gc_collector(Collector_T *recv) {
+static void _gc_mmonit(Mmonit_T *recv) {
 
   ASSERT(recv);
 
   if((*recv)->next)
-    _gc_collector(&(*recv)->next);
+    _gc_mmonit(&(*recv)->next);
 
   _gc_url(&(*recv)->url);
 
