@@ -1,11 +1,8 @@
 /*
- * Copyright (C), 2000-2007 by the monit project group.
- * All Rights Reserved.
+ * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 3.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,6 +11,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
  */
 
 
@@ -30,24 +41,23 @@
 "<head>"\
 "  <title>monit: service manager</title>"\
 "  <style type=\"text/css\">"\
-"  <!--"\
-"  BODY {"\
+"  body {"\
 "	margin: 0;"\
 "  }"\
-"  BODY, P, DIV, TD, TH, TR, FORM, OL, UL, LI, INPUT, TEXTAREA, SELECT, A {"\
-"  	font-family: Tahoma, Arial, Helvetica, sans-serif;"\
+"  body, p, div, td, th, tr, form, ol, ul, li, input, textarea, select, a {"\
+"  	font-family: \"lucida grande\", verdana, geneva, arial, helvetica, sans-serif;"\
 "  	font-size: 14px; "\
 "  }"\
-"  A:hover {"\
+"  a:hover {"\
 "  	text-decoration: none;"\
 "  }"\
-"  A  {"\
+"  a  {"\
 "  	text-decoration: underline;"\
 "  }"\
-"  .grey {"\
+"  .foot {"\
+"        font-size:10px;"\
 "        color: #666666;"\
 "  } "\
-"  -->"\
 "  </style>"\
 "<meta HTTP-EQUIV=\"REFRESH\" CONTENT=%d>"\
 "<meta HTTP-EQUIV=\"Expires\" Content=0>"\
@@ -81,15 +91,9 @@
 
 
 #define FOOT_HTML "</center>"\
-"<p>&nbsp;</p>"\
-"<p>&nbsp;</p>"\
-"<p><br>"\
-"</p>"\
-"<div align=\"center\"><font color=\"#666666\">"\
-"<small>"\
-"&copy; Copyright 2000-2007 by <a class=grey "\
-"href=\"http://www.tildeslash.com/monit/who.html\">the monit project group</a>."\
-" All Rights Reserved. </small></font></div><br></body></html>"
+"<div align=\"center\" class=\"foot\" style=\"margin:20px auto\">"\
+"Copyright &copy; 2000-2009 by <a class=\"foot\" href=\"http://tildeslash.com/\">Tildeslash Ltd</a>."\
+" All Rights Reserved.</div></body></html>"
 
 
 #define HEAD(location, refresh) \
