@@ -1,11 +1,8 @@
 /*
- * Copyright (C), 2000-2007 by the monit project group.
- * All Rights Reserved.
+ * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 3.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,6 +11,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
  */
 
 #include <config.h>
@@ -100,7 +111,7 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Christian Hopp, <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: processor.c,v 1.61 2007/10/11 20:48:06 martinp Exp $
+ *  @version \$Id: processor.c,v 1.66 2009/02/13 09:18:15 hauk Exp $
  *
  *  @file
  */
@@ -181,6 +192,7 @@ void send_error(HttpResponse res, int code, const char *msg) {
 	   "<hr><a href='%s'><font size=-1>%s</font></a>"\
 	   "</body></html>\r\n",
 	    code, err, err, msg?msg:"", SERVER_URL, get_server(server, STRLEN));
+	DEBUG("HttpRequest error: %s %d %s\n", SERVER_PROTOCOL, code, msg ? msg : err);
 }
 
 
@@ -810,6 +822,7 @@ static void internal_error(Socket_T S, int status, char *msg) {
 	       "</body></html>\r\n",
 	       SERVER_PROTOCOL, status, status_msg, date, server,
 	       status_msg, status_msg, msg, SERVER_URL, server);
+  DEBUG("HttpRequest error: %s %d %s\n", SERVER_PROTOCOL, status, msg ? msg : status_msg);
 }
 
 
