@@ -1,11 +1,8 @@
 /*
- * Copyright (C), 2000-2007 by the monit project group.
- * All Rights Reserved.
+ * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 3.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,6 +11,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
  */
 
 #include <config.h>
@@ -73,15 +84,15 @@
 
 
 /**
- *  Implementation of a logger that appends log messages to a file
- *  with a preceding timestamp. Methods support both syslog or own
- *  logfile.
- *
- *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
- *
- *  @version \$Id: log.c,v 1.35 2007/07/26 07:41:32 hauk Exp $
- *
- *  @file
+  *  Implementation of a logger that appends log messages to a file
+  *  with a preceding timestamp. Methods support both syslog or own
+  *  logfile.
+  *
+  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
+  *
+  *  @version \$Id: log.c,v 1.42 2009/02/13 09:18:07 hauk Exp $
+  *
+  *  @file
  */
 
 
@@ -97,14 +108,14 @@ static struct mylogpriority {
   char *description;
 } logPriority[]= {
   {LOG_EMERG,   "emergency"},
-  {LOG_ALERT,   "alert"},
-  {LOG_CRIT,    "critical"},
-  {LOG_ERR,     "error"},
-  {LOG_WARNING, "warning"},
-  {LOG_NOTICE,  "notice"},
-  {LOG_INFO,    "info"},
-  {LOG_DEBUG,   "debug"},
-  {-1,          NULL}
+    {LOG_ALERT,   "alert"},
+    {LOG_CRIT,    "critical"},
+    {LOG_ERR,     "error"},
+    {LOG_WARNING, "warning"},
+    {LOG_NOTICE,  "notice"},
+    {LOG_INFO,    "info"},
+    {LOG_DEBUG,   "debug"},
+    {-1,          NULL}
 };
 
 
@@ -126,20 +137,20 @@ static void log_log(int priority, const char *s, va_list ap);
  */
 int log_init() {
 
-  if(!Run.dolog) {
+  if (!Run.dolog) {
     return TRUE;
   }
 
-  if(!open_log()) {
+  if (!open_log()) {
     return FALSE;
   }
-  
+
   /* Register log_close to be
-     called at program termination */
-  atexit(log_close);
+  called at program termination */
+    atexit(log_close);
 
   return TRUE;
-  
+
 }
 
 
@@ -267,18 +278,17 @@ void LogDebug(const char *s, ...) {
  * Close the log file or syslog
  */
 void log_close() {
-  
-  if(Run.use_syslog) {
+
+  if (Run.use_syslog) {
     closelog(); 
   }
-  
-  if(LOG  && (0 != fclose(LOG))) {
-    LogError("%s: Error closing the log file -- %s\n",
-	prog, STRERROR);
+
+  if (LOG  && (0 != fclose(LOG))) {
+    LogError("%s: Error closing the log file -- %s\n",	prog, STRERROR);
   }
-  
+
   LOG= NULL;
-  
+
 }
 
 
@@ -289,14 +299,13 @@ void log_close() {
  * Open a log file or syslog
  */
 static int open_log() {
- 
-  if(Run.use_syslog) {
+
+  if (Run.use_syslog) {
     openlog(prog, LOG_PID, Run.facility); 
   } else {
     umask(LOGMASK);
-    if((LOG= fopen(Run.logfile,"a+")) == (FILE *)NULL) {
-      LogError("%s: Error opening the log file '%s' for writing -- %s\n",
-	  prog, Run.logfile, STRERROR);
+    if ((LOG= fopen(Run.logfile,"a+")) == (FILE *)NULL) {
+      LogError("%s: Error opening the log file '%s' for writing -- %s\n", prog, Run.logfile, STRERROR);
       return(FALSE);
     }
     /* Set logger in unbuffered mode */
@@ -304,7 +313,7 @@ static int open_log() {
   }
 
   return TRUE;
-  
+
 }
 
 
@@ -316,14 +325,14 @@ static char *timefmt(char *t, int size) {
 
   time_t now;
   struct tm tm;
-  
+
   time(&now);
-  if(!strftime(t, size, TIMEFORMAT, localtime_r(&now, &tm))) {
+  if (!strftime(t, size, TIMEFORMAT, localtime_r(&now, &tm))) {
     *t= 0;
   }
-  
+
   return t;
-  
+
 }
 
 
@@ -337,9 +346,9 @@ static const char *logPriorityDescription(int p) {
 
   struct mylogpriority *lp= logPriority;
 
-  while((*lp).description)
+  while ((*lp).description)
   {
-    if(p == (*lp).priority)
+    if (p == (*lp).priority)
     {
       return (*lp).description;
     }
@@ -363,47 +372,52 @@ static void log_log(int priority, const char *s, va_list ap) {
 #endif
 
   ASSERT(s);
-  
+
   LOCK(log_mutex)
+
 #ifdef HAVE_VA_COPY
     va_copy(ap_copy, ap);
-    vfprintf(stderr, s, ap_copy);
-    va_end(ap_copy);
+  vfprintf(stderr, s, ap_copy);
+  va_end(ap_copy);
 #else
-    vfprintf(stderr, s, ap);
+  vfprintf(stderr, s, ap);
 #endif
-    fflush(stderr);
-  END_LOCK;
-  
-  if(Run.dolog) {
-    char datetime[STRLEN];
+  fflush(stderr);
 
-    if(Run.use_syslog) {
-      LOCK(log_mutex)
+  if (Run.dolog) {
+    if (Run.use_syslog) {
 #ifdef HAVE_VA_COPY
-        va_copy(ap_copy, ap);
-        vsyslog(priority, s, ap_copy);
-        va_end(ap_copy);
+      va_copy(ap_copy, ap);
+      vsyslog(priority, s, ap_copy);
+      va_end(ap_copy);
 #else
-        vsyslog(priority, s, ap);
+      vsyslog(priority, s, ap);
 #endif
-      END_LOCK;
-    } else if(LOG) {
-      LOCK(log_mutex)
-        fprintf(LOG, "[%s] %-8s : ",
-          timefmt(datetime, STRLEN),
-          logPriorityDescription(priority));
+    } else if (LOG) {
+      char datetime[STRLEN];
+      fprintf(LOG, "[%s] %-8s : ", timefmt(datetime, STRLEN), logPriorityDescription(priority));
 #ifdef HAVE_VA_COPY
-        va_copy(ap_copy, ap);
-      	vfprintf(LOG, s, ap_copy);
-        va_end(ap_copy);
+      va_copy(ap_copy, ap);
+      vfprintf(LOG, s, ap_copy);
+      va_end(ap_copy);
 #else
-        vfprintf(LOG, s, ap);
+      vfprintf(LOG, s, ap);
 #endif
-      END_LOCK;
 
     }
   }
+  END_LOCK;
 }
 
 
+#ifndef HAVE_VSYSLOG
+#ifdef HAVE_SYSLOG
+void vsyslog (int facility_priority, const char *format, va_list arglist) {
+  char msg[STRLEN+1];
+
+  vsnprintf(msg, STRLEN, format, arglist);
+  syslog(facility_priority, "%s", msg);
+}
+#endif /* HAVE_SYSLOG */
+#endif /* HAVE_VSYSLOG */
+
