@@ -1,11 +1,8 @@
 /*
- * Copyright (C), 2000-2007 by the monit project group.
- * All Rights Reserved.
+ * Copyright (C) 2000-2009 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 3.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,6 +11,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
  */
 
 
@@ -54,7 +65,7 @@
 #include <regex.h>
 #endif
 
-#ifdef HAVE_ERRNO_H
+#ifdef HAVE_SYSLOG_H
 #include <syslog.h>
 #endif
 
@@ -62,6 +73,10 @@
 #include <limits.h>
 #endif
 
+#ifdef HAVE_SYS_UTSNAME_H
+#include <sys/utsname.h>
+#endif
+
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
 #endif
@@ -79,6 +94,8 @@
 #define HTTP_CONTENT_MAX   (1024*1000)
 /* Set event queue directory mode: "drwx------" */
 #define QUEUEMASK          0077
+/* Set file mode: "drw-------" */
+#define PRIVATEMASK        0177
 /* Set log file mode: "-rw-rw-r--" */
 #define LOGMASK            0112
 /* Set pid file mode: "-rw-r--r--" */
@@ -86,6 +103,7 @@
 #define MYPIDDIR           PIDDIR
 #define MYPIDFILE          "monit.pid"
 #define MYSTATEFILE        "monit.state"
+#define MYIDFILE           "monit.id"
 #define MYEVENTLISTBASE    "/var/monit"
 
 #define LOCALHOST          "localhost"
@@ -97,6 +115,9 @@
 
 #define SSL_TIMEOUT        15
 
+#define START_DELAY        0
+#define EXEC_TIMEOUT       30
+
 #define START_HTTP         1
 #define STOP_HTTP          2
 
@@ -107,9 +128,11 @@
 #define MONITOR_YES        1
 #define MONITOR_INIT       2
 
-#define STATE_PASSED       0
+#define STATE_SUCCEEDED    0
 #define STATE_FAILED       1
-#define STATE_INIT         2
+#define STATE_CHANGED      2
+#define STATE_CHANGEDNOT   3
+#define STATE_INIT         4
 
 #define MODE_ACTIVE        0
 #define MODE_PASSIVE       1
@@ -134,13 +157,14 @@
 #define ACTION_START       6
 #define ACTION_MONITOR     7
 
-#define TYPE_DEVICE        0
+#define TYPE_FILESYSTEM    0
 #define TYPE_DIRECTORY     1
 #define TYPE_FILE          2
 #define TYPE_PROCESS       3
 #define TYPE_HOST          4
 #define TYPE_SYSTEM        5
 #define TYPE_FIFO          6
+#define TYPE_STATUS        7
 
 #define RESOURCE_ID_CPU_PERCENT       1
 #define RESOURCE_ID_MEM_PERCENT       2
@@ -156,10 +180,12 @@
 #define RESOURCE_ID_CPUUSER           12
 #define RESOURCE_ID_CPUSYSTEM         13
 #define RESOURCE_ID_CPUWAIT           14
+#define RESOURCE_ID_TOTAL_CPU_PERCENT 15
 
 #define DIGEST_CLEARTEXT   1
 #define DIGEST_CRYPT       2
 #define DIGEST_MD5         3
+#define DIGEST_PAM         4
 
 #define UNIT_BYTE          1
 #define UNIT_KILOBYTE      1024
@@ -183,10 +209,10 @@
 #define LEVEL_NAME_FULL    "full"
 #define LEVEL_NAME_SUMMARY "summary"
 
-#define HANDLER_PASSED     0x0
+#define HANDLER_SUCCEEDED  0x0
 #define HANDLER_ALERT      0x1
-#define HANDLER_COLLECTOR  0x2
-#define HANDLER_MAX        HANDLER_COLLECTOR
+#define HANDLER_MMONIT     0x2
+#define HANDLER_MAX        HANDLER_MMONIT
 
 #define ICMP_ATTEMPT_COUNT      3         
 #define ICMP_MAX_COUNT          20
@@ -234,8 +260,8 @@ Sigfunc *signal(int signo, Sigfunc * func);
 /** ------------------------------------------ Simple Assert Exception macro */
 
 
-#define ASSERT(e) if(!(e)) { LogCritical("AssertException: " #e \
-        " at %s:%d\naborting..\n", __FILE__, __LINE__); abort(); }
+#define ASSERT(e) do { if(!(e)) { LogCritical("AssertException: " #e \
+        " at %s:%d\naborting..\n", __FILE__, __LINE__); abort(); } } while(0)
 
 
 /* --------------------------------------------------------- Data structures */
@@ -259,7 +285,7 @@ typedef struct mycommand {
   uid_t uid;         /**< The user id to switch to when running this Command */
   int   has_gid;          /**< TRUE if a new gid is defined for this Command */
   gid_t gid;        /**< The group id to switch to when running this Command */
-  unsigned timeout;      /**< Max cycles which we wait for method to execute */
+  unsigned timeout;     /**< Max seconds which we wait for method to execute */
 } *Command_T;
 
 
@@ -276,7 +302,7 @@ typedef struct myaction {
 /** Defines event's up and down actions */
 typedef struct myeventaction {
   Action_T  failed;                  /**< Action in the case of failure down */
-  Action_T  passed;                    /**< Action in the case of failure up */
+  Action_T  succeeded;                    /**< Action in the case of failure up */
 } *EventAction_T;
 
 
@@ -306,15 +332,15 @@ typedef struct myrequest {
 
 
 /** Defines an event notification and status receiver object */
-typedef struct mycollector {
+typedef struct mymmonit {
   URL_T url;                                             /**< URL definition */
   Ssl_T ssl;                                             /**< SSL definition */
   int   timeout;              /**< The timeout to wait for connection or i/o */
   Socket_T       socket;                     /**< Socket used for connection */
 
   /** For internal use */
-  struct mycollector *next;                      /**< next receiver in chain */
-} *Collector_T;
+  struct mymmonit *next;                         /**< next receiver in chain */
+} *Mmonit_T;
 
 
 /** Defines a mailinglist object */
@@ -347,6 +373,7 @@ typedef struct mymailserver {
 typedef struct myauthentication {
   char *uname;                  /**< User allowed to connect to monit httpd */
   char *passwd;                                /**< The users password data */
+  char *groupname;                                      /**< PAM group name */
   int   digesttype;                      /**< How did we store the password */
   int   is_readonly;     /**< TRUE if this is a read-only authenticated user*/
   struct myauthentication *next;       /**< Next credential or NULL if last */
@@ -380,7 +407,8 @@ typedef struct myprocesstree {
 
 /** Defines data for systemwide statistic */
 typedef struct mysysteminfo {
-  time_t collected;                            /**< When were data collected */
+  struct timeval collected;                    /**< When were data collected */
+  int cpus;                                              /**< Number of CPUs */
   double loadavg[3];                                /**< Load average triple */
   unsigned long mem_kbyte_max;               /**< Maximal system real memory */
   unsigned long total_mem_kbyte; /**< Total real memory in use in the system */
@@ -388,6 +416,7 @@ typedef struct mysysteminfo {
   int    total_cpu_user_percent;   /**< Total CPU in use in user space (pct.)*/
   int    total_cpu_syst_percent; /**< Total CPU in use in kernel space (pct.)*/
   int    total_cpu_wait_percent;      /**< Total CPU in use in waiting (pct.)*/
+  struct utsname uname;        /**< Platform information provided by uname() */
 } SystemInfo_T;
 
 
@@ -422,6 +451,7 @@ typedef struct myport {
   int  request_hashtype;  /**< The optional type of hash for a req. document */
   char *pathname;                   /**< Pathname, in case of an UNIX socket */
   char *address;               /**< Human readable destination of the socket */
+  int maxforward;            /**< Optional max forward for protocol checking */
   Generic_T generic;                                /**< Generic test handle */
   int timeout;   /**< The timeout in seconds to wait for connect or read i/o */
   int is_available;                /**< TRUE if the server/port is available */
@@ -512,6 +542,7 @@ typedef struct mysize {
   int  operator;                                    /**< Comparison operator */
   unsigned long long size;                               /**< Size watermark */
   int  test_changes;            /**< TRUE if we only should test for changes */
+  int  test_changes_ok;   /**< TRUE if size was initialized for changes test */
   EventAction_T action;  /**< Description of the action upon event occurence */
   
   /** For internal use */
@@ -521,10 +552,11 @@ typedef struct mysize {
 
 /** Defines checksum object */
 typedef struct mychecksum {
-  char     *hash;                 /**< A checksum hash computed for the path */
-  int       type;                   /**< The type of hash (e.g. md5 or sha1) */
-  int       length;                                  /**< Length of the hash */
-  int       test_changes;       /**< TRUE if we only should test for changes */
+  char *hash;                     /**< A checksum hash computed for the path */
+  int   type;                       /**< The type of hash (e.g. md5 or sha1) */
+  int   length;                                      /**< Length of the hash */
+  int   test_changes;           /**< TRUE if we only should test for changes */
+  int   test_changes_ok; /**< TRUE if cksum was initialized for changes test */
   EventAction_T action;  /**< Description of the action upon event occurence */
 } *Checksum_T;
 
@@ -565,8 +597,8 @@ typedef struct mygid {
 } *Gid_T;
 
 
-/** Defines filesystem device configuration */
-typedef struct mydevice {
+/** Defines filesystem configuration */
+typedef struct myfilesystem {
   int  resource;                        /**< Whether to check inode or space */
   int  operator;                                    /**< Comparison operator */
   long limit_absolute;                               /**< Watermark - blocks */
@@ -574,8 +606,8 @@ typedef struct mydevice {
   EventAction_T action;  /**< Description of the action upon event occurence */
 
   /** For internal use */
-  struct mydevice *next;                           /**< next device in chain */
-} *Device_T;
+  struct myfilesystem *next;                   /**< next filesystem in chain */
+} *Filesystem_T;
 
 
 /** Defines service data */
@@ -588,7 +620,7 @@ typedef struct myinfo {
   ino_t   st_ino;                                                 /**< Inode */
   time_t  timestamp;                                          /**< Timestamp */
 
-  /* Device specific */
+  /* Filesystem specific */
   long   f_bsize;                                   /**< Transfer block size */
   long   f_blocks;                      /**< Total data blocks in filesystem */
   long   f_blocksfree;           /**< Free blocks available to non-superuser */
@@ -654,7 +686,7 @@ typedef struct myservice {
 
   /** Test rules and event handlers */
   Checksum_T  checksum;                                  /**< Checksum check */
-  Device_T    devicelist;                             /**< Device check list */
+  Filesystem_T filesystemlist;                    /**< Filesystem check list */
   Gid_T       gid;                                            /**< Gid check */
   Icmp_T      icmplist;                                 /**< ICMP check list */
   Perm_T      perm;                                    /**< Permission check */
@@ -680,23 +712,26 @@ typedef struct myservice {
   EventAction_T action_MONIT_START;         /**< Monit instance start action */
   EventAction_T action_MONIT_STOP;           /**< Monit instance stop action */
   EventAction_T action_MONIT_RELOAD;       /**< Monit instance reload action */
+  EventAction_T action_ACTION;           /**< Action requested by CLI or GUI */
 
   /** Runtime parameters */
   unsigned long long error;                          /**< Error flags bitmap */
+  unsigned long long error_hint;   /**< Failed/Changed hint for error bitmap */
   Info_T             inf;                          /**< Service check result */
-  time_t             collected;                /**< When were data collected */
+  struct timeval     collected;                /**< When were data collected */
   int                doaction;          /**< Action scheduled by http thread */
+  char              *token;                                /**< Action token */
 
   /** Events */
   struct myevent {
-    #define           EVENT_VERSION  1      /**< The event structure version */
+    #define           EVENT_VERSION  2      /**< The event structure version */
     int               id;                      /**< The event identification */
-    time_t            collected;                 /**< When the event occured */
+    struct timeval    collected;                 /**< When the event occured */
     char             *source;                 /**< Event source service name */
     char             *group;                         /**< Service group name */
     int               mode;             /**< Monitoring mode for the service */
     int               type;                      /**< Monitored service type */
-    short             state;            /**< TRUE if failed, FALSE if passed */
+    short             state;         /**< TRUE if failed, FALSE if succeeded */
     short             state_changed;              /**< TRUE if state changed */
     long long         state_map;           /**< Event bitmap for last cycles */
     unsigned int      count;                             /**< The event rate */
@@ -710,7 +745,7 @@ typedef struct myservice {
   } *eventlist;                                     /**< Pending events list */
 
   /** Context specific parameters */
-  char *path;   /**< Path to the device, file, directory or process pid file */
+  char *path;  /**< Path to the filesys, file, directory or process pid file */
   
   /** For internal use */
   pthread_mutex_t   mutex;        /**< Mutex used for action synchronization */
@@ -730,6 +765,8 @@ struct myrun {
   char *logfile;                         /**< The file to write logdata into */
   char *localhostname;                      /**< The host name for localhost */
   char *pidfile;                                  /**< This programs pidfile */
+  char *idfile;                           /**< The file with unique monit id */
+  char id[STRLEN];                                      /**< Unique monit id */
   char *statefile;                /**< The file with the saved runtime state */
   char *mygroup;                              /**< Group Name of the Service */
   int  debug;                   /**< Write debug information - TRUE or FALSE */
@@ -737,6 +774,7 @@ struct myrun {
   int  dolog;       /**< TRUE if program should log actions, otherwise FALSE */
   int  isdaemon;                 /**< TRUE if program should run as a daemon */
   int  polltime;        /**< In deamon mode, the sleeptime (sec) between run */
+  int  startdelay;                    /**< the sleeptime (sec) after startup */
   int  dohttpd;                    /**< TRUE if monit HTTP server should run */
   int  httpdssl;                     /**< TRUE if monit HTTP server uses ssl */
   char *httpsslpem;                       /**< PEM file for the HTTPS server */
@@ -755,7 +793,6 @@ struct myrun {
   int  doaction;             /**< TRUE if some service(s) has action pending */
   mode_t umask;                /**< The initial umask monit was started with */
   int  testing;   /**< Running in configuration testing mode - TRUE or FALSE */
-  int  wait_start;         /**< Number of threads waiting for process' start */
   time_t incarnation;              /**< Unique ID for running monit instance */
   int  handler_init;                  /**< The handlers queue initialization */
   int  handler_flag;                            /**< The handlers state flag */
@@ -763,6 +800,7 @@ struct myrun {
   Service_T system;                          /**< The general system service */
   char *eventlist_dir;                   /**< The event queue base directory */
   int  eventlist_slots;          /**< The event queue size - number of slots */
+  int  expectbuffer; /**< Generic protocol expect buffer - STRLEN by default */
 
        /** An object holding program relevant "environment" data, see; env.c */
   struct myenvironment {
@@ -771,10 +809,11 @@ struct myrun {
     char *cwd;                                /**< Current working directory */
   } Env;
 
+  char *mail_hostname;    /**< Used in HELO/EHLO/MessageID when sending mail */
   int mailserver_timeout;    /**< Connect and read timeout for a SMTP server */
   Mail_T maillist;                /**< Global alert notification mailinglist */
   MailServer_T mailservers;    /**< List of MTAs used for alert notification */
-  Collector_T collectors;  /**< Event notification and status receivers list */
+  Mmonit_T mmonits;        /**< Event notification and status receivers list */
   Auth_T credentials;    /** A list holding Basic Authentication information */
   Event_T eventlist;              /** A list holding partialy handled events */
                                       /** User selected standard mail format */
@@ -827,11 +866,17 @@ void  LogNotice(const char *, ...);
 void  LogInfo(const char *, ...);
 void  LogDebug(const char *, ...);
 void  log_close();
+#ifndef HAVE_VSYSLOG
+#ifdef HAVE_SYSLOG
+void vsyslog (int, const char *, va_list);
+#endif /* HAVE_SYSLOG */
+#endif /* HAVE_VSYSLOG */
 void  validate() ;
 void  daemonize() ;
 void  gc();
 void  gc_mail_list(Mail_T *);
-void _gccmd(Command_T *);
+void  gccmd(Command_T *);
+void  gc_event(Event_T *e);
 int   kill_daemon(int);
 int   exist_daemon(); 
 int   sendmail(Mail_T);
@@ -854,19 +899,20 @@ void *rpl_malloc (size_t __size);
 void unset_signal_block(sigset_t *);
 void set_signal_block(sigset_t *, sigset_t *);
 int  check_process(Service_T);
-int  check_device(Service_T);
+int  check_filesystem(Service_T);
 int  check_file(Service_T);
 int  check_directory(Service_T);
 int  check_remote_host(Service_T);
 int  check_system(Service_T);
 int  check_fifo(Service_T);
+int  check_status(Service_T);
 int  check_URL(Service_T s);
 int  sha_md5_stream (FILE *, void *, void *);
 void reset_procinfo(Service_T);
 int  check_service_status(Service_T);
 void printhash(char *);  
 char *status_xml(Event_T, short);
-int  handle_collector(Event_T);
+int  handle_mmonit(Event_T);
 int  do_wakeupcall();
 
 #endif
