@@ -1,11 +1,8 @@
 /*
- * Copyright (C), 2000-2007 by the monit project group.
- * All Rights Reserved.
+ * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 3.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,6 +11,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
  */
 
 
@@ -90,7 +101,7 @@
  *  @author Martin Pala <martinp@tildeslash.com>
  *  @author Arkadiusz Miskiewicz <arekm@pld-linux.org>
  *
- *  @version \$Id: sysdep_LINUX.c,v 1.44 2007/07/25 12:54:32 hauk Exp $
+ *  @version \$Id: sysdep_LINUX.c,v 1.53 2009/02/27 20:09:39 martinp Exp $
  *
  *  @file
  */
@@ -135,10 +146,13 @@ int init_process_info_sysdep(void) {
     return FALSE;
   }
 
-  if((num_cpus = sysconf(_SC_NPROCESSORS_CONF)) == -1)
+  if((systeminfo.cpus = sysconf(_SC_NPROCESSORS_CONF)) < 0)
   {
     DEBUG("system statistic error -- cannot get cpu count: %s\n", STRERROR);
     return FALSE;
+  } else if(systeminfo.cpus == 0) {
+    DEBUG("system reports cpu count 0, setting dummy cpu count 1\n");
+    systeminfo.cpus = 1;
   }
 
   if((page_size = sysconf(_SC_PAGESIZE)) <= 0) {
@@ -244,6 +258,7 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
      * alpha system it is 1/1024s */
     pt[i].cputime =
         ((float)(stat_item_utime + stat_item_stime) * 10.0) / HZ;
+    pt[i].cpu_percent = 0;
 
     /* State is Zombie -> then we are a Zombie ... clear or? (-: */
     if(stat_item_state == 'Z')
@@ -398,9 +413,9 @@ int used_system_cpu_sysdep(SystemInfo_T *si) {
 
   if(old_cpu_total == 0)
   {
-    si->total_cpu_user_percent = 0;
-    si->total_cpu_syst_percent = 0;
-    si->total_cpu_wait_percent = 0;
+    si->total_cpu_user_percent = -10;
+    si->total_cpu_syst_percent = -10;
+    si->total_cpu_wait_percent = -10;
   }
   else
   {
