@@ -1,11 +1,8 @@
 /*
- * Copyright (C), 2000-2007 by the monit project group.
- * All Rights Reserved.
+ * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 3.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,6 +11,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
  */
 
 
@@ -89,7 +100,7 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: sysdep_SOLARIS.c,v 1.21 2007/07/25 12:54:32 hauk Exp $
+ *  @version \$Id: sysdep_SOLARIS.c,v 1.28 2009/02/13 09:18:18 hauk Exp $
  *
  *  @file
  */
@@ -111,7 +122,7 @@ int init_process_info_sysdep(void) {
 
   register int pagesize;
 
-  num_cpus= sysconf( _SC_NPROCESSORS_ONLN);
+  systeminfo.cpus= sysconf( _SC_NPROCESSORS_ONLN);
 
   pagesize = sysconf(_SC_PAGESIZE);
   pageshift = 0;
@@ -185,6 +196,7 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
     if (!read_proc_file(buf,4096, "psinfo", pt[i].pid)) {
       
       pt[i].cputime = 0;
+      pt[i].cpu_percent = 0;
       pt[i].mem_kbyte= 0;
       continue;
 
@@ -203,6 +215,7 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
       /* and does it anyway matter? */
       
       pt[i].cputime = 0;
+      pt[i].cpu_percent = 0;
       pt[i].mem_kbyte= 0;
       continue;
       
@@ -213,11 +226,13 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
     if (!read_proc_file(buf,4096, "status", pt[i].pid)) {
       
       pt[i].cputime=0;
+      pt[i].cpu_percent = 0;
       
     } else {
       
       pt[i].cputime= ( timestruc_to_tseconds(pstatus->pr_utime) +
                        timestruc_to_tseconds(pstatus->pr_stime) );
+      pt[i].cpu_percent = 0;
     }
     
   }
@@ -364,9 +379,9 @@ int used_system_cpu_sysdep(SystemInfo_T *si) {
 
   if ( old_total == 0.0 ) {
 
-    si->total_cpu_user_percent=0;
-    si->total_cpu_syst_percent=0;
-    si->total_cpu_wait_percent=0;
+    si->total_cpu_user_percent=-10;
+    si->total_cpu_syst_percent=-10;
+    si->total_cpu_wait_percent=-10;
     
   } else {
 
