@@ -1,11 +1,8 @@
 /*
- * Copyright (C), 2000-2007 by the monit project group.
- * All Rights Reserved.
+ * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 3.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,6 +11,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
  */
 
 
@@ -66,7 +77,7 @@
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *
- *  @version \$Id: sendmail.c,v 1.55 2007/11/06 20:32:30 martinp Exp $
+ *  @version \$Id: sendmail.c,v 1.67 2009/03/24 19:49:06 martinp Exp $
  *
  *  @file
  */
@@ -101,10 +112,11 @@ static void do_send(SendMail_T *S, const char *, ...);
 /**
  * Send mail messages via SMTP
  * @param mail A Mail object
- * @return FALSE if failed, TRUE if passed
+ * @return FALSE if failed, TRUE if succeeded
  */
 int sendmail(Mail_T mail) {
 
+  int i;
   int rv;
   Mail_T m;
   SendMail_T S;
@@ -113,6 +125,7 @@ int sendmail(Mail_T mail) {
   
   ASSERT(mail);
   
+  S.socket = NULL;
   if((rv = sigsetjmp(S.error, TRUE))) {
     rv = FALSE;
     goto exit;
@@ -124,8 +137,8 @@ int sendmail(Mail_T mail) {
   
   Util_getRFC822Date(NULL, now, STRLEN);
   
-  if(gethostname(S.localhost, sizeof(S.localhost)) < 0) {
-    snprintf(S.localhost, sizeof(S.localhost), "%s", LOCALHOST);
+  if(Run.mail_hostname || gethostname(S.localhost, sizeof(S.localhost)) < 0) {
+    snprintf(S.localhost, sizeof(S.localhost), "%s", Run.mail_hostname?Run.mail_hostname:LOCALHOST);
   }
   
   do_status(&S);
@@ -159,7 +172,7 @@ int sendmail(Mail_T mail) {
     do_status(&S);
   }
   
-  for(m= mail; m; m= m->next) { 
+  for(i = 0, m= mail; m; m= m->next, i++) { 
     do_send(&S, "MAIL FROM: <%s>\r\n", m->from);
     do_status(&S);
     do_send(&S, "RCPT TO: <%s>\r\n", m->to);
@@ -173,8 +186,8 @@ int sendmail(Mail_T mail) {
     do_send(&S, "X-Mailer: %s %s\r\n", prog, VERSION);
     do_send(&S, "Mime-Version: 1.0\r\n");
     do_send(&S, "Content-Type: text/plain; charset=\"iso-8859-1\"\r\n");
-    do_send(&S, "Content-Transfer-Encoding: quoted-printable\r\n");
-		do_send(&S, "Message-id: <%ld@%s>\r\n", time(NULL), S.localhost);
+    do_send(&S, "Content-Transfer-Encoding: 8bit\r\n");
+    do_send(&S, "Message-id: <%ld.%d@%s>\r\n", time(NULL), i, S.localhost);
     do_send(&S, "\r\n");
     do_send(&S, "%s\r\n", m->message);
     do_send(&S, ".\r\n");
@@ -187,8 +200,7 @@ exit:
   if(S.socket)
     socket_free(&S.socket);
   
-  if(b64)
-    FREE(b64);
+  FREE(b64);
 
   return rv;
 }
@@ -253,11 +265,8 @@ static void open_server(SendMail_T *S) {
     S->password    = mta->password;
     S->ssl         = mta->ssl;
   } else {
-    S->server      = LOCALHOST;
-    S->port        = PORT_SMTP;
-    S->username    = NULL;
-    S->password    = NULL;
-    S->ssl.use_ssl = FALSE;
+    LogError("No mail servers are defined -- see manual for 'set mailserver' statement\n");
+    siglongjmp(S->error, TRUE);
   }
   
   do {
