@@ -1,11 +1,8 @@
 /*
- * Copyright (C), 2000-2007 by the monit project group.
- * All Rights Reserved.
+ * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 3.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,6 +11,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
  */
 
 #include <config.h>
@@ -76,7 +87,7 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Peter Holdaway <pholdaway@technocom-wireless.com>
  *
- *  @version \$Id: spawn.c,v 1.50 2007/07/25 12:54:30 hauk Exp $
+ *  @version \$Id: spawn.c,v 1.57 2009/02/13 09:18:10 hauk Exp $
  *  @file
  */
 
@@ -235,7 +246,6 @@ void spawn(Service_T S, Command_T C, const char *E) {
    * We do not need to wait for the second child since we forked twice,
    * the init system-process will wait for it. So we just return
    */
-  return ;
 
 } 
 
@@ -267,6 +277,11 @@ static void set_monit_environment(Service_T s,
 
   snprintf(buf, STRLEN, "MONIT_EVENT=%s", event?event:"No Event");
   push_monit_environment(buf, e);
+  
+  if (s->eventlist) {
+    snprintf(buf, STRLEN, "MONIT_DESCRIPTION=%s", s->eventlist->message);
+    push_monit_environment(buf, e);
+  }
 
   if (s->type == TYPE_PROCESS) {
     snprintf(buf, STRLEN, "MONIT_PROCESS_PID=%d", Util_isProcessRunning(s));
