@@ -1,11 +1,8 @@
 /*
- * Copyright (C), 2000-2007 by the monit project group.
- * All Rights Reserved.
+ * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 3.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,6 +11,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
  */
 
 
@@ -144,7 +155,7 @@ static pthread_mutex_t *ssl_mutex_table;
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: ssl.c,v 1.61 2007/07/25 12:54:30 hauk Exp $
+ *  @version \$Id: ssl.c,v 1.66 2009/02/13 09:18:10 hauk Exp $
  *
  *  @file
  */
@@ -969,17 +980,10 @@ static void cleanup_ssl_socket(ssl_connection *ssl) {
     ssl->socket_bio= NULL;
   }
 
-  if(ssl->cert_issuer)
-    FREE(ssl->cert_issuer);
-
-  if(ssl->cert_subject)
-    FREE(ssl->cert_subject);
-
-  if(ssl->cert_md5)
-    FREE(ssl->cert_md5);
-
-  if(ssl->clientpemfile)
-    FREE(ssl->clientpemfile);
+  FREE(ssl->cert_issuer);
+  FREE(ssl->cert_subject);
+  FREE(ssl->cert_md5);
+  FREE(ssl->clientpemfile);
 
 }
 
@@ -993,11 +997,8 @@ static void cleanup_ssl_server_socket(ssl_server_connection *ssl_server) {
   if(!ssl_server)
     return;
 
-  if(ssl_server->pemfile)
-    FREE(ssl_server->pemfile);
-
-  if(ssl_server->clientpemfile)
-    FREE(ssl_server->clientpemfile);
+  FREE(ssl_server->pemfile);
+  FREE(ssl_server->clientpemfile);
 
   while(ssl_server->ssl_conn_list) {
     ssl_connection *ssl = ssl_server->ssl_conn_list;
