@@ -1,11 +1,8 @@
 /*
- * Copyright (C), 2000-2007 by the monit project group.
- * All Rights Reserved.
+ * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 3.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,6 +11,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
  */
 
 #include <config.h>
@@ -57,13 +68,13 @@
 /**
  *  Manage service information persistently. Service data is saved to
  *  a state file when monit runs in daemon mode for each poll
- *  cycle. Monit use this file to recover from a crash or to maintain
- *  service data persistently during a reload. The location of the
- *  state file may be set from the command line or set in the monitrc
- *  file, if not set, the default is ~/.monit.state.
+ *  cycle. Monit use this file to maintain service data persistently
+ *  during reload or restart. The location of the state file may be
+ * set from the command line or set in the monitrc file, if not set,
+ * the default is ~/.monit.state.
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
- *  @version \$Id: state.c,v 1.30 2007/07/25 12:54:30 hauk Exp $
+ *  @version \$Id: state.c,v 1.37 2009/02/13 09:18:10 hauk Exp $
  *  @file
  */
 
@@ -106,13 +117,10 @@ void State_save() {
   Service_T s;
   State_T state;
   FILE *S= NULL;
-  sigset_t ns,os;
   
   if(! (S= open_state("w")))
       return;
 
-  set_signal_block(&ns, &os);
-
   l= Util_getNumberOfServices();
   
   if(fwrite(&l, 1, sizeof (int), S) != sizeof(int)) {
@@ -132,8 +140,6 @@ void State_save() {
   
   error:
   close_state(S);
-  unset_signal_block(&os);
-  
 }
 
 
@@ -184,15 +190,12 @@ void State_update() {
   int l= 0;
   State_T s;
   FILE *S= NULL;
-  sigset_t ns, os;
   Service_T service;
   int has_error= FALSE;
   
   if(! (S= open_state("r")))
       return;
   
-  set_signal_block(&ns, &os);
-
   errno= 0;
   if(fread(&l, 1, sizeof (int), S) != sizeof(int)) {
     LogError("%s: Unable to read monit state information from '%s'\n",
@@ -217,7 +220,6 @@ void State_update() {
 
   error:
   close_state(S);
-  unset_signal_block(&os);
 
   if(!has_error)
       State_save();
@@ -269,7 +271,6 @@ static void clone_state(Service_T service, State_T *state) {
   state->nstart= service->nstart;
   state->ncycle= service->ncycle;
   state->monitor= service->monitor;
-  state->error= service->error;
 
 }
 
@@ -280,6 +281,5 @@ static void update_service_state(Service_T service, State_T *state) {
   service->nstart= state->nstart;
   service->ncycle= state->ncycle;
   service->monitor= state->monitor;
-  service->error= state->error;
   
 }
