@@ -1,11 +1,8 @@
 /*
- * Copyright (C), 2000-2007 by the monit project group.
- * All Rights Reserved.
+ * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 3.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,6 +11,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
  */
 
 #include <config.h>
@@ -93,6 +104,22 @@
 #include <arpa/inet.h>
 #endif
 
+#ifdef HAVE_SECURITY_PAM_APPL_H
+#include <security/pam_appl.h>
+#endif
+
+#ifdef HAVE_PAM_PAM_APPL_H
+#include <pam/pam_appl.h>
+#endif
+
+#ifdef HAVE_PWD_H
+#include <pwd.h>
+#endif
+
+#ifdef HAVE_GRP_H
+#include <grp.h>
+#endif
+
 #include "monitor.h"
 #include "engine.h"
 #include "md5.h"
@@ -104,10 +131,25 @@
 
 
 /* Private prototypes */
-static char x2c(char *hex);
-static char *is_str_defined(char *);
-static void printevents(unsigned int);
-static int is_url_unsafe(unsigned char *);
+static char   x2c(char *hex);
+static char  *is_str_defined(char *);
+static void   printevents(unsigned int);
+static int    is_url_unsafe(unsigned char *);
+#ifdef HAVE_LIBPAM
+#ifdef SOLARIS
+static int    PAMquery(int, struct pam_message **, struct pam_response **, void *);
+#else
+static int    PAMquery(int, const struct pam_message **, struct pam_response **, void *);
+#endif
+static int    PAMcheckPasswd(const char *, const char *);
+static Auth_T PAMcheckUserGroup(const char *);
+#endif
+
+
+struct ad_user {
+  const char *login;
+  const char *passwd;
+};
 
 
 /**
@@ -118,7 +160,7 @@ static int is_url_unsafe(unsigned char *);
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *  @author Michael Amster, <mamster@webeasy.com> 
  *
- *  @version \$Id: util.c,v 1.199 2007/10/16 03:05:41 hauk Exp $
+ *  @version \$Id: util.c,v 1.231 2009/04/11 20:06:01 martinp Exp $
  *  @file
  */
 
@@ -258,6 +300,21 @@ void Util_trimQuotes(char *s) {
 }
 
 
+char *Util_trunc(char *s, int n) {
+        ASSERT(n>=0);
+        if (s) {
+                int sl = strlen(s);
+                if (sl > (n + 4)) {
+                        int e = n+3;
+                        for (; n < e; n++)
+                                s[n]= '.';
+                        s[n]= 0;
+                }
+        }
+        return s;
+}
+
+
 /**
  * Replace all occurrences of the <code>old</code> char in the string
  * <code>s</code> with the <code>new</code> char.
@@ -582,7 +639,8 @@ void Util_printRunList() {
   printf(" %-18s = %s\n", "Use syslog", Run.use_syslog?"True":"False");
   printf(" %-18s = %s\n", "Is Daemon", Run.isdaemon?"True":"False");
   printf(" %-18s = %s\n", "Use process engine", Run.doprocess?"True":"False");
-  printf(" %-18s = %d seconds\n", "Poll time", Run.polltime);
+  printf(" %-18s = %d seconds with start delay %d seconds\n", "Poll time", Run.polltime, Run.startdelay);
+  printf(" %-18s = %d bytes\n", "Expect buffer", Run.expectbuffer);
 
   if(Run.eventlist_dir) {
     char slots[STRLEN];
@@ -596,16 +654,16 @@ void Util_printRunList() {
       "Event queue", Run.eventlist_dir, slots);
   }
 
-  if(Run.collectors) {
-    Collector_T c= Run.collectors;
-    printf(" %-18s = ", "Collector(s)");
-    for(c= Run.collectors; c; c= c->next)
+  if(Run.mmonits) {
+    Mmonit_T c= Run.mmonits;
+    printf(" %-18s = ", "M/Monit(s)");
+    for(c= Run.mmonits; c; c= c->next)
     {
       printf("%s with timeout %d seconds%s%s%s%s%s%s",
         c->url->url,
         c->timeout,
-	c->ssl.use_ssl?" ssl version ":"",
-	c->ssl.use_ssl?sslnames[c->ssl.version]:"",
+	(c->ssl.use_ssl && c->ssl.version) ? " ssl version " : "",
+	(c->ssl.use_ssl && c->ssl.version) ? sslnames[c->ssl.version] : "",
 	c->ssl.certmd5?" server cert md5 sum ":"",
 	c->ssl.certmd5?c->ssl.certmd5:"",
         c->url->user?" using credentials":"",
@@ -614,8 +672,8 @@ void Util_printRunList() {
     printf("\n");
   }
 
-  printf(" %-18s = ", "Mail server(s)");
   if(Run.mailservers) {
+    printf(" %-18s = ", "Mail server(s)");
     MailServer_T mta= Run.mailservers;
     for(mta= Run.mailservers; mta; mta= mta->next)
 	printf("%s:%d%s%s",
@@ -623,9 +681,10 @@ void Util_printRunList() {
           mta->port,
           mta->ssl.use_ssl?"(ssl)":"",
           mta->next?", ":" ");
+    printf("with timeout %d seconds", Run.mailserver_timeout);
+    if(Run.mail_hostname)
+      printf(" using '%s' as my hostname", Run.mail_hostname);
     printf("\n");
-  } else {
-    printf("localhost\n");
   }
 
   printf(" %-18s = %s\n", "Mail from", is_str_defined(Run.MailFormat.from));
@@ -695,7 +754,7 @@ void Util_printService(Service_T s) {
   Port_T n;
   Icmp_T i;
   Mail_T r;
-  Device_T dl;
+  Filesystem_T dl;
   Resource_T q;
   Timestamp_T t;
   Size_T sl;
@@ -729,7 +788,7 @@ void Util_printService(Service_T s) {
       printf(" as uid %d", s->start->uid);
     if(s->start->has_gid)
       printf(" as gid %d", s->start->gid);
-    printf(" timeout %d cycle(s)", s->start->timeout);
+    printf(" timeout %d second(s)", s->start->timeout);
     printf("\n");
   }
   if(s->stop) {
@@ -745,7 +804,7 @@ void Util_printService(Service_T s) {
       printf(" as uid %d", s->stop->uid);
     if(s->stop->has_gid)
       printf(" as gid %d", s->stop->gid);
-    printf(" timeout %d cycle(s)", s->stop->timeout);
+    printf(" timeout %d second(s)", s->stop->timeout);
     printf("\n");
   }
 
@@ -765,7 +824,7 @@ void Util_printService(Service_T s) {
       s->action_PPID->failed->description);
   }
 
-  if(s->type == TYPE_DEVICE) {
+  if(s->type == TYPE_FILESYSTEM) {
     printf(" %-20s = if changed %s then %s\n",
       "Filesystem flags",
       Util_getEventratio(s->action_FSFLAG->failed, ratio1),
@@ -777,53 +836,53 @@ void Util_printService(Service_T s) {
     EventAction_T a= cs->action;
 
     Util_getEventratio(a->failed, ratio1);
-    Util_getEventratio(a->passed, ratio2);
+    Util_getEventratio(a->succeeded, ratio2);
 
     if(cs->test_changes)
       printf(" %-20s = if changed %s %s then %s\n",
         "Checksum", checksumnames[cs->type],
         ratio1, a->failed->description);
     else
-      printf(" %-20s = if failed %s(%s) %s then %s else if passed %s then %s\n",
+      printf(" %-20s = if failed %s(%s) %s then %s else if succeeded %s then %s\n",
         "Checksum", cs->hash, checksumnames[cs->type],
         ratio1, a->failed->description,
-        ratio2, a->passed->description);
+        ratio2, a->succeeded->description);
   }
   
   if(s->perm && s->perm->action) {
     EventAction_T a= s->perm->action;
 
     Util_getEventratio(a->failed, ratio1);
-    Util_getEventratio(a->passed, ratio2);
+    Util_getEventratio(a->succeeded, ratio2);
 
-    printf(" %-20s = if failed %04o %s then %s else if passed %s then %s\n",
+    printf(" %-20s = if failed %04o %s then %s else if succeeded %s then %s\n",
       "Permission", s->perm->perm,
       ratio1, a->failed->description,
-      ratio2, a->passed->description);
+      ratio2, a->succeeded->description);
   }
 
   if(s->uid && s->uid->action) {
     EventAction_T a= s->uid->action;
 
     Util_getEventratio(a->failed, ratio1);
-    Util_getEventratio(a->passed, ratio2);
+    Util_getEventratio(a->succeeded, ratio2);
 
-    printf(" %-20s = if failed %d %s then %s else if passed %s then %s\n",
+    printf(" %-20s = if failed %d %s then %s else if succeeded %s then %s\n",
       "UID", (int)s->uid->uid,
       ratio1, a->failed->description,
-      ratio2, a->passed->description);
+      ratio2, a->succeeded->description);
   }
 
   if(s->gid && s->gid->action) {
     EventAction_T a= s->gid->action;
 
     Util_getEventratio(a->failed, ratio1);
-    Util_getEventratio(a->passed, ratio2);
+    Util_getEventratio(a->succeeded, ratio2);
 
-    printf(" %-20s = if failed %d %s then %s else if passed %s then %s\n",
+    printf(" %-20s = if failed %d %s then %s else if succeeded %s then %s\n",
       "GID", (int)s->gid->gid,
       ratio1, a->failed->description,
-      ratio2, a->passed->description);
+      ratio2, a->succeeded->description);
   }
 
   if(s->icmplist)
@@ -831,13 +890,13 @@ void Util_printService(Service_T s) {
       EventAction_T a= i->action;
 
       Util_getEventratio(a->failed, ratio1);
-      Util_getEventratio(a->passed, ratio2);
+      Util_getEventratio(a->succeeded, ratio2);
 
       printf(" %-20s = if failed %s count %d with timeout %d seconds %s then %s "
-        "else if passed %s then %s\n",
+        "else if succeeded %s then %s\n",
         "ICMP", icmpnames[i->type], i->count, i->timeout,
         ratio1, a->failed->description,
-        ratio2, a->passed->description);
+        ratio2, a->succeeded->description);
     }
 
   if(s->portlist) {
@@ -846,17 +905,17 @@ void Util_printService(Service_T s) {
       EventAction_T a= n->action;
       
       Util_getEventratio(a->failed, ratio1);
-      Util_getEventratio(a->passed, ratio2);
+      Util_getEventratio(a->succeeded, ratio2);
 
       if(n->family == AF_INET) {
 	
 	   printf(" %-20s = if failed %s:%d%s [%s via %s] with "
-             "timeout %d seconds %s then %s else if passed %s then %s\n",
+             "timeout %d seconds %s then %s else if succeeded %s then %s\n",
 	     "Port", n->hostname, n->port, n->request?n->request:"",
              n->protocol->name, Util_portTypeDescription(n),
              n->timeout,
              ratio1, a->failed->description,
-             ratio2, a->passed->description);
+             ratio2, a->succeeded->description);
 	  
 	   if(n->SSL.certmd5 != NULL)
 	     printf(" %-20s = %s\n", "Server cert md5 sum", n->SSL.certmd5);
@@ -864,10 +923,10 @@ void Util_printService(Service_T s) {
       } else if(n->family == AF_UNIX) {
 	
         printf(" %-20s = if failed %s [protocol %s] with timeout %d seconds "
-          "%s then %s else if passed %s then %s\n",
+          "%s then %s else if succeeded %s then %s\n",
           "Unix Socket", n->pathname, n->protocol->name, n->timeout,
           ratio1, a->failed->description,
-          ratio2, a->passed->description);
+          ratio2, a->succeeded->description);
 	
       }
       
@@ -879,17 +938,17 @@ void Util_printService(Service_T s) {
     EventAction_T a= t->action;
 
     Util_getEventratio(a->failed, ratio1);
-    Util_getEventratio(a->passed, ratio2);
+    Util_getEventratio(a->succeeded, ratio2);
 
     if(t->test_changes)
       printf(" %-20s = if changed %s then %s\n",
         "Timestamp",
         ratio1, a->failed->description);
     else
-      printf(" %-20s = if %s %d second(s) %s then %s else if passed %s then %s\n",
+      printf(" %-20s = if %s %d second(s) %s then %s else if succeeded %s then %s\n",
         "Timestamp", operatornames[t->operator], t->time,
         ratio1, a->failed->description,
-        ratio2, a->passed->description);
+        ratio2, a->succeeded->description);
     
   }
 
@@ -897,17 +956,17 @@ void Util_printService(Service_T s) {
     EventAction_T a= sl->action;
     
     Util_getEventratio(a->failed, ratio1);
-    Util_getEventratio(a->passed, ratio2);
+    Util_getEventratio(a->succeeded, ratio2);
 
     if(sl->test_changes)
       printf(" %-20s = if changed %s then %s\n",
         "Size",
         ratio1, a->failed->description);
     else
-      printf(" %-20s = if %s %llu byte(s) %s then %s else if passed %s then %s\n",
+      printf(" %-20s = if %s %llu byte(s) %s then %s else if succeeded %s then %s\n",
         "Size", operatornames[sl->operator], sl->size,
         ratio1, a->failed->description,
-        ratio2, a->passed->description);
+        ratio2, a->succeeded->description);
     
   }
 
@@ -922,46 +981,46 @@ void Util_printService(Service_T s) {
     
   }
   
-  for(dl= s->devicelist; dl; dl= dl->next) {
+  for(dl= s->filesystemlist; dl; dl= dl->next) {
     EventAction_T a= dl->action;
     
     Util_getEventratio(a->failed, ratio1);
-    Util_getEventratio(a->passed, ratio2);
+    Util_getEventratio(a->succeeded, ratio2);
 
     if(dl->resource == RESOURCE_ID_INODE) {
 	    
       if(dl->limit_absolute > -1) {
-        printf(" %-20s = if %s %ld %s then %s else if passed %s then %s\n",
+        printf(" %-20s = if %s %ld %s then %s else if succeeded %s then %s\n",
           "Inodes usage limit",
           operatornames[dl->operator],
           dl->limit_absolute,
           ratio1, a->failed->description,
-          ratio2, a->passed->description);
+          ratio2, a->succeeded->description);
       } else {
-        printf(" %-20s = if %s %.1f%% %s then %s else if passed %s then %s\n",
+        printf(" %-20s = if %s %.1f%% %s then %s else if succeeded %s then %s\n",
           "Inodes usage limit",
           operatornames[dl->operator],
           dl->limit_percent/10.,
           ratio1, a->failed->description,
-          ratio2, a->passed->description);
+          ratio2, a->succeeded->description);
       }
       
     } else if(dl->resource == RESOURCE_ID_SPACE) {
 	    
       if(dl->limit_absolute > -1) {
-        printf(" %-20s = if %s %ld blocks %s then %s else if passed %s then %s\n",
+        printf(" %-20s = if %s %ld blocks %s then %s else if succeeded %s then %s\n",
           "Space usage limit",
           operatornames[dl->operator],
           dl->limit_absolute,
           ratio1, a->failed->description,
-          ratio2, a->passed->description);
+          ratio2, a->succeeded->description);
       } else {
-        printf(" %-20s = if %s %.1f%% %s then %s else if passed %s then %s\n",
+        printf(" %-20s = if %s %.1f%% %s then %s else if succeeded %s then %s\n",
           "Space usage limit",
           operatornames[dl->operator],
           dl->limit_percent/10.,
           ratio1, a->failed->description,
-          ratio2, a->passed->description);
+          ratio2, a->succeeded->description);
       }
 
     }
@@ -972,127 +1031,137 @@ void Util_printService(Service_T s) {
     EventAction_T a= q->action;
 
     Util_getEventratio(a->failed, ratio1);
-    Util_getEventratio(a->passed, ratio2);
+    Util_getEventratio(a->succeeded, ratio2);
 
     switch(q->resource_id) {
 
     case RESOURCE_ID_CPU_PERCENT: 
 
       printf(" %-20s = if %s %.1f%% %s then %s "
-        "else if passed %s then %s\n", 
+        "else if succeeded %s then %s\n", 
         "CPU usage limit", 
         operatornames[q->operator], q->limit/10.0,
         ratio1, a->failed->description,
-        ratio2, a->passed->description);
+        ratio2, a->succeeded->description);
+      break;
+
+    case RESOURCE_ID_TOTAL_CPU_PERCENT: 
+
+      printf(" %-20s = if %s %.1f%% %s then %s "
+        "else if succeeded %s then %s\n", 
+        "CPU usage limit (incl. children)", 
+        operatornames[q->operator], q->limit/10.0,
+        ratio1, a->failed->description,
+        ratio2, a->succeeded->description);
       break;
 
     case RESOURCE_ID_CPUUSER: 
 
       printf(" %-20s = if %s %.1f%% %s then %s "
-        "else if passed %s then %s\n", 
+        "else if succeeded %s then %s\n", 
         "CPU user limit", 
         operatornames[q->operator], q->limit/10.0,
         ratio1, a->failed->description,
-        ratio2, a->passed->description);
+        ratio2, a->succeeded->description);
       break;
 
     case RESOURCE_ID_CPUSYSTEM: 
 
       printf(" %-20s = if %s %.1f%% %s then %s "
-        "else if passed %s then %s\n", 
+        "else if succeeded %s then %s\n", 
         "CPU system limit", 
         operatornames[q->operator], q->limit/10.0,
         ratio1, a->failed->description,
-        ratio2, a->passed->description);
+        ratio2, a->succeeded->description);
       break;
 
     case RESOURCE_ID_CPUWAIT: 
 
       printf(" %-20s = if %s %.1f%% %s then %s "
-        "else if passed %s then %s\n", 
+        "else if succeeded %s then %s\n", 
         "CPU wait limit", 
         operatornames[q->operator], q->limit/10.0,
         ratio1, a->failed->description,
-        ratio2, a->passed->description);
+        ratio2, a->succeeded->description);
       break;
 
     case RESOURCE_ID_MEM_PERCENT: 
 
       printf(" %-20s = if %s %.1f%% %s then %s "
-        "else if passed %s then %s\n", 
+        "else if succeeded %s then %s\n", 
         "Memory usage limit", 
         operatornames[q->operator], q->limit/10.0,
         ratio1, a->failed->description,
-        ratio2, a->passed->description);
+        ratio2, a->succeeded->description);
       break;
 
     case RESOURCE_ID_MEM_KBYTE: 
 
       printf(" %-20s = if %s %ldkB %s then %s "
-        "else if passed %s then %s\n", 
+        "else if succeeded %s then %s\n", 
         "Memory amount limit", 
         operatornames[q->operator], q->limit,
         ratio1, a->failed->description,
-        ratio2, a->passed->description);
+        ratio2, a->succeeded->description);
       break;
 
     case RESOURCE_ID_LOAD1: 
 
       printf(" %-20s = if %s %.1f %s then %s "
-        "else if passed %s then %s\n", 
+        "else if succeeded %s then %s\n", 
         "Load avg. (1min)", 
         operatornames[q->operator], q->limit/10.0,
         ratio1, a->failed->description,
-        ratio2, a->passed->description);
+        ratio2, a->succeeded->description);
       break;
 
     case RESOURCE_ID_LOAD5: 
 
       printf(" %-20s = if %s %.1f %s then %s "
-        "else if passed %s then %s\n", 
+        "else if succeeded %s then %s\n", 
         "Load avg. (5min)", 
         operatornames[q->operator], q->limit/10.0,
         ratio1, a->failed->description,
-        ratio2, a->passed->description);
+        ratio2, a->succeeded->description);
       break;
 
     case RESOURCE_ID_LOAD15: 
 
       printf(" %-20s = if %s %.1f %s then %s "
-        "else if passed %s then %s\n", 
+        "else if succeeded %s then %s\n", 
         "Load avg. (15min)", 
         operatornames[q->operator], q->limit/10.0,
         ratio1, a->failed->description,
-        ratio2, a->passed->description);
+        ratio2, a->succeeded->description);
       break;
 
     case RESOURCE_ID_CHILDREN:
 
       printf(" %-20s = If %s %ld %s then %s "
-        "else if passed %s then %s\n",
+        "else if succeeded %s then %s\n",
         "Children", operatornames[q->operator], q->limit,
         ratio1, a->failed->description,
-        ratio2, a->passed->description);
+        ratio2, a->succeeded->description);
       break;
 
     case RESOURCE_ID_TOTAL_MEM_KBYTE:
 
       printf(" %-20s = If %s %ld %s then %s "
-        "else if passed %s then %s\n",
+        "else if succeeded %s then %s\n",
         "Memory amount limit (incl. children)",
         operatornames[q->operator], q->limit,
         ratio1, a->failed->description,
-        ratio2, a->passed->description);
+        ratio2, a->succeeded->description);
       break;
 
     case RESOURCE_ID_TOTAL_MEM_PERCENT:
 
       printf(" %-20s = If %s %.1f%% %s then %s "
-        "else if passed %s then %s\n",
+        "else if succeeded %s then %s\n",
         "Memory usage limit (incl. children)",
         operatornames[q->operator], q->limit/10.0,
         ratio1, a->failed->description,
-        ratio2, a->passed->description);
+        ratio2, a->succeeded->description);
       break;
 
     }    
@@ -1104,12 +1173,12 @@ void Util_printService(Service_T s) {
   if(s->def_timeout && s->action_TIMEOUT) {
     EventAction_T a= s->action_TIMEOUT;
     printf(" %-20s = If %d restart within %d cycles then %s "
-      "else if passed then %s\n",
+      "else if succeeded then %s\n",
       "Timeout",
       s->to_start,
       s->to_cycle,
       a->failed->description,
-      a->passed->description);
+      a->succeeded->description);
   }
 
   for(r= s->maillist; r; r= r->next) {
@@ -1195,6 +1264,53 @@ fileerror:
   exit(1);
 }
 
+/**
+ * Open and read the id from the given idfile. If the idfile doesn't exist,
+ * generate new id and store it in the id file.
+ * @param idfile An idfile with full path
+ * @return the id
+ */
+char *Util_monitId(char *idfile) {
+  FILE *file = NULL;
+
+  ASSERT(idfile);
+
+  if(! File_exist(idfile)) {
+    char buf[STRLEN];
+    unsigned char digest[STRLEN];
+    mode_t mask = umask(PRIVATEMASK);
+    file = fopen(idfile, "w");
+    umask(mask);
+    if(! file) {
+      LogError("%s: Error opening the idfile '%s' -- %s\n", prog, idfile, STRERROR);
+      return NULL;
+    }
+    /* Generate the unique id */
+    srandom(time(NULL) + getpid());
+    snprintf(buf, STRLEN, "%lu%d%lu", (unsigned long)time(NULL), getpid(), random());
+    md5_buffer(buf, strlen(buf), digest);
+    Util_digest2Bytes(digest, 16, Run.id);
+    fprintf(file, "%s", Run.id);
+    LogInfo("%s: generated unique Monit id %s and stored to '%s'\n", prog, Run.id, idfile);
+  } else {
+    if(! File_isFile(idfile)) {
+      LogError("%s: idfile '%s' is not a regular file\n", prog, idfile);
+      return NULL;
+    }
+    if((file= fopen(idfile,"r")) == (FILE *)NULL) {
+      LogError("%s: Error opening the idfile '%s' -- %s\n", prog, idfile, STRERROR);
+      return NULL;
+    }
+    if(fscanf(file, "%s", Run.id) != 1) {
+      LogError("%s: Error reading id from file '%s'\n", prog, idfile);
+      return NULL;
+    }
+  }
+  fclose(file);
+
+  return Run.id;
+}
+
 /**
  * Open and read the pid from the given pidfile.
  * @param pidfile A pidfile with full path
@@ -1202,7 +1318,6 @@ fileerror:
  * not be read from the file
  */
 pid_t Util_getPid(char *pidfile) {
-  
   FILE *file= NULL;
   int pid= -1;
 
@@ -1217,19 +1332,19 @@ pid_t Util_getPid(char *pidfile) {
     return FALSE;
   }
   if((file= fopen(pidfile,"r")) == (FILE *)NULL) {
-    LogError("%s: Error opening the pidfile '%s' -- %s\n",
-	prog, pidfile, STRERROR);
-    return FALSE ;
+    LogError("%s: Error opening the pidfile '%s' -- %s\n", prog, pidfile, STRERROR);
+    return FALSE;
+  }
+  if(fscanf(file, "%d", &pid) != 1) {
+    LogError("%s: Error reading pid from file '%s'\n", prog, pidfile);
+    return FALSE;
   }
-
-  fscanf(file, "%d", &pid);
   fclose(file);
 
-  if(pid == -1) {
-    return (FALSE);
-  }
+  if(pid < 0)
+    return(FALSE);
   
-  return (pid_t)pid;
+  return(pid_t)pid;
   
 }
 
@@ -1249,7 +1364,7 @@ int Util_isProcessRunning(Service_T s) {
     if( (getpgid(pid) > -1) || (errno == EPERM) )
       return pid;
     DEBUG("'%s' Error testing process id [%d] -- %s\n", s->name, pid, STRERROR);
-	}
+  }
   Util_resetInfo(s);
   
   return FALSE;
@@ -1457,41 +1572,44 @@ char *Util_urlDecode(char *url) {
  * @return a Basic Authentication Authorization string (RFC 2617),
  * with credentials from the Run object, NULL if credentials are not defined.
  */
-char *Util_getBasicAuthHeader() {
+char *Util_getBasicAuthHeaderMonit() {
 
-  Auth_T c= Run.credentials;
+  Auth_T c = Run.credentials;
 
-  if (c==NULL) {
-    return NULL;
-  }
-  
   /* We find the first cleartext credential for authorization */
-  while (c!= NULL) {
-    if (c->digesttype == DIGEST_CLEARTEXT) {
+  while (c != NULL) {
+    if (c->digesttype == DIGEST_CLEARTEXT)
       break;
-    }
-    c=c->next;
-  }
-  if(c!=NULL) {
-    char *auth, *b64;
-    char  buf[STRLEN];
-    snprintf(buf, STRLEN, "%s:%s",
-             c->uname,
-             c->passwd);
-    if(! (b64= encode_base64(strlen(buf), (unsigned char *)buf)) ) {
-      LogError("Failed to base64 encode authentication header\n");
-      FREE(b64);
-      return NULL;
-    }
-    auth= xcalloc(sizeof(char), STRLEN+1);
-    snprintf(auth, STRLEN, "Authorization: Basic %s\r\n", b64);
-    FREE(b64);
-    return auth;
+    c = c->next;
   }
 
-  LogError("Cleartext credentials needed for basic authorization!\n");
+  if (c)
+    return Util_getBasicAuthHeader(c->uname, c->passwd);
+
   return NULL;
+}
 
+
+/**
+ * @return a Basic Authentication Authorization string (RFC 2617),
+ * NULL if username is not defined.
+ */
+char *Util_getBasicAuthHeader(char *username, char *password) {
+  char *auth, *b64;
+  char  buf[STRLEN];
+
+  if (!username)
+    return NULL;
+  
+  snprintf(buf, STRLEN, "%s:%s", username, password ? password : "");
+  if(! (b64= encode_base64(strlen(buf), (unsigned char *)buf)) ) {
+      LogError("Failed to base64 encode authentication header\n");
+      return NULL;
+  }
+  auth= xcalloc(sizeof(char), STRLEN+1);
+  snprintf(auth, STRLEN, "Authorization: Basic %s\r\n", b64);
+  FREE(b64);
+  return auth;
 }
 
 
@@ -1578,7 +1696,7 @@ void Util_redirectStdFds() {
 }
 
 
-/*
+/**
  * Close all filedescriptors except standard. Everything
  * seems to have getdtablesize, so we'll use it here, and back
  * out to use 1024 if getdtablesize not available.
@@ -1596,20 +1714,146 @@ void Util_closeFds() {
 }
 
 
-/*
+/**
  * Check if monit does have credentials for this user.  If successful
  * a pointer to the password is returned.
  */
 Auth_T Util_getUserCredentials(char *uname) {
-  Auth_T c= Run.credentials;
-  while ( c != NULL ) {
-    if ( strcmp(c->uname, uname) == 0 ) {
+  Auth_T c;
+
+  /* check allowed user names */
+  for (c = Run.credentials; c; c = c->next)
+    if (c->uname && IS(c->uname, uname))
       return c;
+
+#ifdef HAVE_LIBPAM
+  /* check allowed group names */
+  return(PAMcheckUserGroup(uname));
+#else
+  return NULL;
+#endif
+}
+
+
+#ifdef HAVE_LIBPAM
+/**
+ * PAM conversation
+ */
+#ifdef SOLARIS
+static int PAMquery(int num_msg, struct pam_message **msg, struct pam_response **resp, void *appdata_ptr) {
+#else
+static int PAMquery(int num_msg, const struct pam_message **msg, struct pam_response **resp, void *appdata_ptr) {
+#endif
+  int i;
+  struct ad_user *user = (struct ad_user *)appdata_ptr;
+  struct pam_response *response;
+
+  /* Sanity checking */
+  if (!msg || !resp || !user )
+    return PAM_CONV_ERR;
+
+  response = xcalloc(sizeof(struct pam_response), num_msg);
+
+  for (i = 0; i < num_msg; i++) {
+    response[i].resp = NULL;
+    response[i].resp_retcode = 0;
+
+    switch ((*(msg[i])).msg_style) {
+    case PAM_PROMPT_ECHO_ON:
+      /* Store the login as the response. This likely never gets called, since login was on pam_start() */
+      response[i].resp= appdata_ptr ? xstrdup(user->login) : NULL;
+      break;
+
+    case PAM_PROMPT_ECHO_OFF:
+      /* Store the password as the response */
+      response[i].resp= appdata_ptr ? xstrdup(user->passwd) : NULL;
+      break;
+
+    case PAM_TEXT_INFO:
+    case PAM_ERROR_MSG:
+      /* Shouldn't happen since we have PAM_SILENT set. If it happens anyway, ignore it. */
+      break;
+
+    default:
+      /* Something strange... */
+      if (response != NULL)
+        FREE(response);
+      return PAM_CONV_ERR;
     }
-    c=c->next;
+  }
+  /* On success, return the response structure */
+  *resp = response;
+  return PAM_SUCCESS;
+}
+
+
+/**
+ * Validate login/passwd via PAM service "monit"
+ */
+static int PAMcheckPasswd(const char *login, const char *passwd) {
+  int rv;
+  pam_handle_t *pamh = NULL;
+  struct ad_user user_info = {
+    login,
+    passwd
+  };
+  struct pam_conv conv = {
+    PAMquery,
+    &user_info
+  };
+
+  if ((rv = pam_start("monit", login, &conv, &pamh) != PAM_SUCCESS)) {
+    DEBUG("PAM authentication start failed -- %d\n", rv);
+    return FALSE;
+  }
+
+  rv = pam_authenticate(pamh, PAM_SILENT);
+
+  if (pam_end(pamh, rv) != PAM_SUCCESS)
+     pamh = NULL;
+
+  return(rv == PAM_SUCCESS ? TRUE : FALSE);
+}
+
+
+/**
+ * Check whether the user is member of allowed groups
+ */
+static Auth_T PAMcheckUserGroup(const char *uname) {
+  Auth_T c = Run.credentials;
+  struct passwd *pwd = NULL; 
+  struct group  *grp = NULL;
+
+  ASSERT(uname);
+
+  if (!(pwd = getpwnam(uname)))
+    return NULL;
+
+  if (!(grp = getgrgid(pwd->pw_gid)))
+    return NULL;
+
+  while (c) {
+    if (c->groupname) {
+      struct group *sgrp = NULL;
+
+      /* check for primary group match */
+      if (IS(c->groupname, grp->gr_name))
+        return c;
+
+      /* check secondary groups match */
+      if ((sgrp = getgrnam(c->groupname))) {
+        char **g = NULL;
+
+        for (g = sgrp->gr_mem; *g; g++)
+          if (IS(*g, uname))
+            return c;
+      }
+    }
+    c = c->next;
   }
   return NULL;
 }
+#endif
 
 
 /**
@@ -1627,28 +1871,30 @@ int Util_checkCredentials(char *uname, char *outside) {
     return FALSE;
   }
   switch (c->digesttype) {
-  case DIGEST_CLEARTEXT:
-    {
+    case DIGEST_CLEARTEXT:
       strncpy(outside_crypt, outside, STRLEN); 
       break;
-    }   
-  case DIGEST_MD5:
+    case DIGEST_MD5:
     {
       char id[STRLEN];
       char salt[STRLEN];
-      char * temp;
+      char *temp;
       /* A password looks like this,
        *   $id$salt$digest
        * the '$' around the id are still part of the id. 
        */
       strncpy(id, c->passwd, STRLEN);
-      temp= strchr(id+1, '$');
-      ASSERT(temp);
+      if(! (temp= strchr(id+1, '$'))) {
+        LogError("Password not in MD5 format.\n");
+	return FALSE;
+      }
       temp += 1;
       *temp= '\0';
       strncpy(salt, c->passwd+strlen(id), STRLEN);
-      temp= strchr(salt, '$');
-      ASSERT(temp);
+      if(! (temp= strchr(salt, '$'))) {
+        LogError("Password not in MD5 format.\n");
+	return FALSE;
+      }
       *temp= '\0';
       if (md5_crypt(outside, id, salt, outside_crypt, STRLEN) == NULL) {
 	LogError("Cannot generate MD5 digest error.\n");
@@ -1656,7 +1902,7 @@ int Util_checkCredentials(char *uname, char *outside) {
       }
       break;
     }
-  case DIGEST_CRYPT:
+    case DIGEST_CRYPT:
     {
       char salt[3];
       char * temp;
@@ -1665,9 +1911,14 @@ int Util_checkCredentials(char *uname, char *outside) {
       strncpy(outside_crypt, temp, STRLEN); 
       break;
     }
-  default:
-    LogError("Unknown password digestion method.\n");
-    return FALSE;
+#ifdef HAVE_LIBPAM
+    case DIGEST_PAM:
+      return PAMcheckPasswd(uname, outside);
+      break;
+#endif
+    default:
+      LogError("Unknown password digestion method.\n");
+      return FALSE;
   }
 
   if (strcmp(outside_crypt,c->passwd)==0) {
@@ -1751,8 +2002,7 @@ process_partial_block:
  * Reset the service information structure
  */
 void Util_resetInfo(Service_T s) {
-  if(s->inf->cs_sum)
-    FREE(s->inf->cs_sum);
+  FREE(s->inf->cs_sum);
   memset(s->inf, 0, sizeof *(s->inf));
   s->inf->_pid=        -1;
   s->inf->_ppid=       -1;
@@ -1779,24 +2029,18 @@ int Util_hasServiceStatus(Service_T s) {
 
 /**
  * Construct a HTTP/1.1 Host header utilizing information from the
- * socket. The returned hostBuf is set to "hostname:port" or to the
- * empty string if information is not available or not applicable.
+ * socket. The returned hostBuf is set to "hostname" or "hostname:port".
+ * if port is not equal to the default HTTP port number
  * @param s A connected socket 
  * @param hostBuf the buffer to write the host-header to
  * @param len Length of the hostBuf
  * @return the hostBuffer
  */
 char *Util_getHTTPHostHeader(Socket_T s, char *hostBuf, int len) {
-  if(! strcmp(LOCALHOST, socket_get_remote_host(s)) ||
-     inet_aton(socket_get_remote_host(s), NULL)) {
-    *hostBuf= 0;
-  } else {
-    if(socket_get_remote_port(s)==80)
-       snprintf(hostBuf, len, "%s", socket_get_remote_host(s));
-     else
-       snprintf(hostBuf, len, "%s:%d", socket_get_remote_host(s),
-                socket_get_remote_port(s));
-  }
+  if(socket_get_remote_port(s)==80)
+    snprintf(hostBuf, len, "%s", socket_get_remote_host(s));
+  else
+    snprintf(hostBuf, len, "%s:%d", socket_get_remote_host(s), socket_get_remote_port(s));
   return hostBuf;
 }
 
@@ -1869,6 +2113,8 @@ void Util_monitorUnset(Service_T s) {
   s->nstart= 0;
   s->ncycle= 0;
   s->error = EVENT_NULL;
+  if(s->eventlist)
+    gc_event(&s->eventlist);
   Util_resetInfo(s);
 }
 
@@ -1986,16 +2232,20 @@ static void printevents(unsigned int events) {
   } else if(events == EVENT_ALL) {
     printf("All events");
   } else {
-    if(IS_EVENT_SET(events, EVENT_CHANGED))
-      printf("Change ");
+    if(IS_EVENT_SET(events, EVENT_ACTION))
+      printf("Action ");
     if(IS_EVENT_SET(events, EVENT_CHECKSUM))
       printf("Checksum ");
     if(IS_EVENT_SET(events, EVENT_CONNECTION))
       printf("Connection ");
+    if(IS_EVENT_SET(events, EVENT_CONTENT))
+      printf("Content ");
     if(IS_EVENT_SET(events, EVENT_DATA))
       printf("Data ");
     if(IS_EVENT_SET(events, EVENT_EXEC))
       printf("Exec ");
+    if(IS_EVENT_SET(events, EVENT_FSFLAG))
+      printf("Fsflags ");
     if(IS_EVENT_SET(events, EVENT_GID))
       printf("Gid ");
     if(IS_EVENT_SET(events, EVENT_ICMP))
@@ -2004,12 +2254,14 @@ static void printevents(unsigned int events) {
       printf("Instance ");
     if(IS_EVENT_SET(events, EVENT_INVALID))
       printf("Invalid ");
-    if(IS_EVENT_SET(events, EVENT_MATCH))
-      printf("Match ");
     if(IS_EVENT_SET(events, EVENT_NONEXIST))
       printf("Nonexist ");
     if(IS_EVENT_SET(events, EVENT_PERMISSION))
       printf("Permission ");
+    if(IS_EVENT_SET(events, EVENT_PID))
+      printf("PID ");
+    if(IS_EVENT_SET(events, EVENT_PPID))
+      printf("PPID ");
     if(IS_EVENT_SET(events, EVENT_RESOURCE))
       printf("Resource ");
     if(IS_EVENT_SET(events, EVENT_SIZE))
