@@ -1,11 +1,8 @@
 /*
- * Copyright (C), 2000-2007 by the monit project group.
- * All Rights Reserved.
+ * Copyright (C) 2009 Tildeslash Ltd. All rights reserved.
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 3.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,6 +11,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library under certain conditions as described in each
+ * individual source file, and distribute linked combinations
+ * including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.  If you delete this exception statement from all source
+ * files in the program, then also delete it here.
  */
 
 
@@ -49,7 +60,7 @@
  *
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: xml.c,v 1.29 2007/07/25 12:54:30 hauk Exp $
+ *  @version \$Id: xml.c,v 1.55 2009/03/28 10:37:43 martinp Exp $
  *
  *  @file
  */
@@ -128,14 +139,22 @@ static void document_head(Buffer_T *B) {
    "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\r\n"
    "<monit>\r\n"
    "\t<server>\r\n"
+   "\t\t<id>%s</id>\r\n"
    "\t\t<incarnation>%ld</incarnation>\r\n"
    "\t\t<version>%s</version>\r\n"
    "\t\t<uptime>%ld</uptime>\r\n"
-   "\t\t<poll>%d</poll>\r\n",
+   "\t\t<poll>%d</poll>\r\n"
+   "\t\t<startdelay>%d</startdelay>\r\n"
+   "\t\t<localhostname>%s</localhostname>\r\n"
+   "\t\t<controlfile>%s</controlfile>\r\n",
+   Run.id,
    Run.incarnation,
    VERSION,
    (long)Util_getProcessUptime(Run.pidfile),
-   Run.polltime);
+   Run.polltime,
+   Run.startdelay,
+   Run.localhostname ? Run.localhostname : "",
+   Run.controlfile ? Run.controlfile : "");
 
   if(Run.dohttpd)
   {
@@ -151,7 +170,21 @@ static void document_head(Buffer_T *B) {
   }
  
   buf_print(B,
-   "\t</server>\r\n");
+   "\t</server>\r\n"
+   "\t<platform>\r\n"
+   "\t\t<name>%s</name>\r\n"
+   "\t\t<release>%s</release>\r\n"
+   "\t\t<version>%s</version>\r\n"
+   "\t\t<machine>%s</machine>\r\n"
+   "\t\t<cpu>%d</cpu>\r\n"
+   "\t\t<memory>%lu</memory>\r\n"
+   "\t</platform>\r\n",
+   systeminfo.uname.sysname,
+   systeminfo.uname.release,
+   systeminfo.uname.version,
+   systeminfo.uname.machine,
+   systeminfo.cpus,
+   systeminfo.mem_kbyte_max);
 
 }
 
@@ -174,28 +207,43 @@ static void document_foot(Buffer_T *B) {
  * @param L Status information level
  */
 static void status_service(Service_T S, Buffer_T *B, short L) {
+  Event_T E = S->eventlist;
   buf_print(B,
 	    "\t<service type=\"%d\">\r\n"
-	    "\t\t<collected>%ld</collected>\r\n"
+	    "\t\t<collected_sec>%ld</collected_sec>\r\n"
+	    "\t\t<collected_usec>%ld</collected_usec>\r\n"
 	    "\t\t<name>%s</name>\r\n"
 	    "\t\t<status>%llu</status>\r\n"
+	    "\t\t<status_hint>%llu</status_hint>\r\n"
 	    "\t\t<monitor>%d</monitor>\r\n"
+	    "\t\t<monitormode>%d</monitormode>\r\n"
 	    "\t\t<pendingaction>%d</pendingaction>\r\n"
 	    "\t\t<group>%s</group>\r\n",
 	    S->type,
-	    S->collected,
+	    S->collected.tv_sec,
+	    S->collected.tv_usec,
 	    S->name?S->name:"",
 	    S->error,
+	    S->error_hint,
 	    S->monitor,
+	    S->mode,
 	    S->doaction,
 	    S->group?S->group:"");
+  /* if the service is in error state, display first active error message to provide more details */
+  while (E) {
+    if ((E->state == STATE_FAILED || E->state == STATE_CHANGED) && (S->error & E->id) && E->message) {
+       buf_print(B, "\t\t<status_message>%s</status_message>\r\n", E->message);
+       break;
+    }
+    E = E->next;
+  } 
   if(L == LEVEL_FULL)
   {
     if(Util_hasServiceStatus(S)) {
       if(S->type == TYPE_FILE || 
          S->type == TYPE_DIRECTORY ||
          S->type == TYPE_FIFO ||
-         S->type == TYPE_DEVICE) {
+         S->type == TYPE_FILESYSTEM) {
         buf_print(B,
   		"\t\t<mode>%o</mode>\r\n"
   		"\t\t<uid>%d</uid>\r\n"
@@ -221,28 +269,28 @@ static void status_service(Service_T S, Buffer_T *B, short L) {
   		  checksumnames[S->checksum->type], S->inf->cs_sum);
         }
       }
-      if(S->type == TYPE_DEVICE) {
+      if(S->type == TYPE_FILESYSTEM) {
         buf_print(B,
   		"\t\t<flags>%ld</flags>\r\n"
   		"\t\t<block>\r\n"
-  		"\t\t\t<size>%ld</size>\r\n"
-  		"\t\t\t<total>%ld</total>\r\n"
-  		"\t\t\t<free>%ld</free>\r\n"
-  		"\t\t\t<freetotal>%ld</freetotal>\r\n"
+  		"\t\t\t<percent>%.1f</percent>\r\n"
+  		"\t\t\t<usage>%.1f MB</usage>\r\n"
+                "\t\t\t<total>%.1f MB</total>\r\n"
   		"\t\t</block>\r\n",
   		S->inf->flags,
-  		S->inf->f_bsize,
-  		S->inf->f_blocks,
-  		S->inf->f_blocksfree,
-  		S->inf->f_blocksfreetotal);
+  		S->inf->space_percent/10.,
+  		(float)S->inf->space_total / (float)1048576 * (float)S->inf->f_bsize,
+                (float)S->inf->f_blocks / (float)1048576 * (float)S->inf->f_bsize);
         if(S->inf->f_files > 0) {
           buf_print(B,
   		  "\t\t<inode>\r\n"
-		  "\t\t\t<total>%ld</total>\r\n"
-		  "\t\t\t<free>%ld</free>\r\n"
+                  "\t\t\t<percent>%.1f</percent>\r\n"
+                  "\t\t\t<usage>%ld</usage>\r\n"
+                  "\t\t\t<total>%ld</total>\r\n"
 		  "\t\t</inode>\r\n",
-		  S->inf->f_files,
-		  S->inf->f_filesfree);
+		  S->inf->inode_percent/10.,
+		  S->inf->inode_total,
+                  S->inf->f_files);
         }
       }
       if(S->type == TYPE_PROCESS) {
@@ -257,20 +305,20 @@ static void status_service(Service_T S, Buffer_T *B, short L) {
           buf_print(B,
   		  "\t\t<children>%d</children>\r\n"
   		  "\t\t<memory>\r\n"
-  		  "\t\t\t<kilobyte>%ld</kilobyte>\r\n"
-  		  "\t\t\t<kilobytetotal>%ld</kilobytetotal>\r\n"
   		  "\t\t\t<percent>%.1f</percent>\r\n"
   		  "\t\t\t<percenttotal>%.1f</percenttotal>\r\n"
+  		  "\t\t\t<kilobyte>%ld</kilobyte>\r\n"
+  		  "\t\t\t<kilobytetotal>%ld</kilobytetotal>\r\n"
   		  "\t\t</memory>\r\n"
   		  "\t\t<cpu>\r\n"
 		  "\t\t\t<percent>%.1f</percent>\r\n"
 		  "\t\t\t<percenttotal>%.1f</percenttotal>\r\n"
 		  "\t\t</cpu>\r\n",
 		  S->inf->children,
-		  S->inf->mem_kbyte,
-		  S->inf->total_mem_kbyte,
 		  S->inf->mem_percent/10.0,
 	  	  S->inf->total_mem_percent/10.0,
+  		  S->inf->mem_kbyte,
+  		  S->inf->total_mem_kbyte,
   		  S->inf->cpu_percent/10.0,
   		  S->inf->total_cpu_percent/10.0);
         }
@@ -294,17 +342,17 @@ static void status_service(Service_T S, Buffer_T *B, short L) {
             buf_print(B,
   		    "\t\t<port>\r\n"
   		    "\t\t\t<hostname>%s</hostname>\r\n"
-  		    "\t\t\t<port>%d</port>\r\n"
+  		    "\t\t\t<portnumber>%d</portnumber>\r\n"
   		    "\t\t\t<request>%s</request>\r\n"
-  		    "\t\t\t<type>%s</type>\r\n"
   		    "\t\t\t<protocol>%s</protocol>\r\n"
+  		    "\t\t\t<type>%s</type>\r\n"
   		    "\t\t\t<responsetime>%.3f</responsetime>\r\n"
   		    "\t\t</port>\r\n",
   		    p->hostname?p->hostname:"",
   		    p->port,
   		    p->request?p->request:"",
-  		    Util_portTypeDescription(p),
   		    p->protocol->name?p->protocol->name:"",
+  		    Util_portTypeDescription(p),
   		    p->is_available?p->response:-1.);
   	  
           } else if(p->family == AF_UNIX) {
@@ -321,33 +369,35 @@ static void status_service(Service_T S, Buffer_T *B, short L) {
         }
       }
       if(S->type == TYPE_SYSTEM && Run.doprocess) {
-        buf_print(B,
-                  "\t\t<load>\r\n"
-                  "\t\t\t<avg01>%.2f</avg01>\r\n"
-                  "\t\t\t<avg05>%.2f</avg05>\r\n"
-                  "\t\t\t<avg15>%.2f</avg15>\r\n"
-                  "\t\t</load>\r\n"
-                  "\t\t<cpu>\r\n"
-                  "\t\t\t<user>%.1f</user>\r\n"
-                  "\t\t\t<system>%.1f</system>\r\n"
-                #ifdef HAVE_CPU_WAIT
-                  "\t\t\t<wait>%.1f</wait>\r\n"
-                #endif
-                  "\t\t</cpu>\r\n"
-                  "\t\t<memory>\r\n"
-                  "\t\t\t<kilobyte>%ld</kilobyte>\r\n"
-                  "\t\t\t<percent>%.1f</percent>\r\n"
-                  "\t\t</memory>\r\n",
-                  systeminfo.loadavg[0],
-                  systeminfo.loadavg[1],
-                  systeminfo.loadavg[2],
-                  systeminfo.total_cpu_user_percent/10.,
-                  systeminfo.total_cpu_syst_percent/10.,
-                #ifdef HAVE_CPU_WAIT
-                  systeminfo.total_cpu_wait_percent/10.,
-                #endif
-                  systeminfo.total_mem_kbyte,
-                  systeminfo.total_mem_percent/10.);
+				buf_print(B,
+					"\t\t<system>\r\n"
+					"\t\t\t<load>\r\n"
+					"\t\t\t\t<avg01>%.2f</avg01>\r\n"
+					"\t\t\t\t<avg05>%.2f</avg05>\r\n"
+					"\t\t\t\t<avg15>%.2f</avg15>\r\n"
+					"\t\t\t</load>\r\n"
+					"\t\t\t<cpu>\r\n"
+					"\t\t\t\t<user>%.1f</user>\r\n"
+					"\t\t\t\t<system>%.1f</system>\r\n"
+#ifdef HAVE_CPU_WAIT
+				        "\t\t\t\t<wait>%.1f</wait>\r\n"
+#endif
+					"\t\t\t</cpu>\r\n"
+					"\t\t\t<memory>\r\n"
+					"\t\t\t\t<percent>%.1f</percent>\r\n"
+					"\t\t\t\t<kilobyte>%ld</kilobyte>\r\n"
+					"\t\t\t</memory>\r\n"
+					"\t\t</system>\r\n",
+					systeminfo.loadavg[0],
+					systeminfo.loadavg[1],
+					systeminfo.loadavg[2],
+					systeminfo.total_cpu_user_percent/10.,
+					systeminfo.total_cpu_syst_percent/10.,
+					#ifdef HAVE_CPU_WAIT
+					systeminfo.total_cpu_wait_percent/10.,
+					#endif
+					systeminfo.total_mem_percent/10.,
+					systeminfo.total_mem_kbyte);
       }
     }
   }
@@ -362,20 +412,29 @@ static void status_service(Service_T S, Buffer_T *B, short L) {
  */
 static void status_event(Event_T E, Buffer_T *B) {
 
+  Service_T s;
+  struct timeval *tv;
+
   ASSERT(E);
 
+  if(!(s = Event_get_source(E)))
+    return;
+
+  tv = Event_get_collected(E);
+
   buf_print(B,
     "\t<event>\r\n"
-    "\t\t<collected>%ld</collected>\r\n"
+    "\t\t<collected_sec>%ld</collected_sec>\r\n"
+    "\t\t<collected_usec>%ld</collected_usec>\r\n"
     "\t\t<service>%s</service>\r\n"
     "\t\t<type>%d</type>\r\n"
     "\t\t<group>%s</group>\r\n"
     "\t\t<id>%d</id>\r\n"
     "\t\t<state>%d</state>\r\n"
     "\t\t<action>%d</action>\r\n"
-    "\t\t<message>%s</message>\r\n"
-    "\t</event>\r\n",
-    Event_get_collected(E),
+    "\t\t<message>%s</message>\r\n",
+    tv->tv_sec,
+    tv->tv_usec,
     Event_get_source_name(E),
     Event_get_source_type(E),
     Event_get_source_group(E),
@@ -383,6 +442,13 @@ static void status_event(Event_T E, Buffer_T *B) {
     Event_get_state(E),
     Event_get_action(E),
     Event_get_message(E));
+  if (s->token) {
+    buf_print(B,
+      "\t\t<token>%s</token>\r\n",
+      s->token);
+  }
+  buf_print(B,
+    "\t</event>\r\n");
 }
 
 
